@@ -82,6 +82,9 @@
 #include "t1itype1.h"
 #endif
 
+#define ufst_emprintf(m,s) { outflush(m); emprintf(m, s); outflush(m); }
+#define ufst_emprintf1(m,s,d) { outflush(m); emprintf1(m, s, d); outflush(m); }
+
 #if UFST_VERSION_MAJOR >= 6 && UFST_VERSION_MINOR >= 2
 #undef true
 #undef false
@@ -117,7 +120,12 @@ EXTERN IF_STATE if_state;
 static fapi_ufst_server *static_server_ptr_for_ufst_callback = 0;
 #endif
 
+#if PCLEO_RDR  &&  BYTEORDER==LOHI
 GLOBAL UW16 PCLswapHdr(FSP LPUB8 p, UW16 gifct);        /* UFST header doesn't define it. */
+#define UFST_PCLswapHdr(p, gifct) PCLswapHdr(FSP p, gifct)
+#else
+#define UFST_PCLswapHdr(p, gifct)
+#endif
 
 typedef struct pcleo_glyph_list_elem_s pcleo_glyph_list_elem;
 struct pcleo_glyph_list_elem_s
@@ -291,13 +299,13 @@ open_UFST(fapi_ufst_server * r, const byte * server_param,
             bPlugIn = TRUE;
         }
         else if (gs_debug_c('1'))
-            emprintf(r->mem, "Warning: Unknown UFST parameter ignored.\n");
+            ufst_emprintf(r->mem, "Warning: Unknown UFST parameter ignored.\n");
     }
 #if !NO_SYMSET_MAPPING
     if (!bSSdir) {
         strcpy(ufst_root_dir, ".");
         if (gs_debug_c('1'))
-            emprintf(r->mem,
+            ufst_emprintf(r->mem,
                      "Warning: UFST_SSdir is not specified, will search *.ss files in the curent directory.\n");
     }
 #endif
@@ -314,7 +322,7 @@ open_UFST(fapi_ufst_server * r, const byte * server_param,
     }
     else {
 #ifdef FCO_RDR
-        emprintf(r->mem,
+        ufst_emprintf(r->mem,
                  "Warning: UFST_PlugIn is not specified, some characters may be missing.\n");
 #endif
     }
@@ -335,7 +343,7 @@ gs_fapi_ufst_ensure_open(gs_fapi_server *server, const char *server_param, int s
     {
         code = open_UFST(r, (byte *)server_param, server_param_size);
         if (code < 0) {
-            emprintf(r->mem, "Error opening the UFST font server.\n");
+            ufst_emprintf(r->mem, "Error opening the UFST font server.\n");
             return code;
         }
     }
@@ -890,7 +898,7 @@ ufst_make_font_data(fapi_ufst_server * r, const char *font_file_path,
         return (gs_error_invalidaccess);
 #else
         /* If we have the Freetype server available, always use it for non-FCO fonts */
-        if (gs_fapi_available(r->mem, (char *)"Freetype")) {
+        if (gs_fapi_available(r->mem, (char *)"FreeType")) {
             return (gs_error_invalidaccess);
         }
 
@@ -930,7 +938,7 @@ ufst_make_font_data(fapi_ufst_server * r, const char *font_file_path,
             return (gs_error_invalidaccess);
 #else
             /* If we have the Freetype server available, always use it for non-FCO fonts */
-            if (gs_fapi_available(r->mem, (char *)"Freetype")) {
+            if (gs_fapi_available(r->mem, (char *)"FreeType")) {
                 return (gs_error_invalidaccess);
             }
 #endif
@@ -962,10 +970,10 @@ ufst_make_font_data(fapi_ufst_server * r, const char *font_file_path,
             d->font_type = FC_FCO_TYPE;
         }
         else {
-            stream *f = sfopen(font_file_path, "rb", r->mem);
+            stream *f = sfopen(font_file_path, "r", r->mem);
 
             if (f == NULL) {
-                emprintf1(r->mem,
+                ufst_emprintf1(r->mem,
                           "fapiufst: Can't open %s\n", font_file_path);
                 return gs_error_undefinedfilename;
             }
@@ -988,9 +996,7 @@ ufst_make_font_data(fapi_ufst_server * r, const char *font_file_path,
             d->font_id = assign_font_id();
         }
         h = (PCLETTO_FHDR *) (buf + sizeof(ufst_common_font_data));
-        /* For some reason the bytes of fontDescriptorSize need swapped here
-           (PCLswapHdr() doesn't do it as needed), but only for TTF fonts - duh!?!?
-         */
+
         if (ff->is_type1) {
             h->fontDescriptorSize = PCLETTOFONTHDRSIZE;
         }
@@ -1047,7 +1053,8 @@ ufst_make_font_data(fapi_ufst_server * r, const char *font_file_path,
         /*  fixme : This code assumes 1-byte alignment for PCLETTO_FHDR structure.
            Use PACK_* macros to improve.
          */
-        PCLswapHdr(FSA(UB8 *) h, 0);
+        UFST_PCLswapHdr((UB8 *) h, 0);
+
         if (ff->is_type1) {
             LPUB8 fontdata = (LPUB8) h + PCLETTOFONTHDRSIZE;
 
@@ -1158,9 +1165,9 @@ gs_fapi_ufst_get_scaled_font_aux(gs_fapi_server * server, gs_fapi_font * ff,
         hy *= 256;
 
         while (hx < 1.5 || hy < 1.5) {
-            world_scale += 8;
-            hx *= 256;
-            hy *= 256;
+            world_scale += 1;
+            hx *= 10;
+            hy *= 10;
         }
     }
     fc->s.m2.world_scale = world_scale;
@@ -1330,7 +1337,7 @@ gs_fapi_ufst_get_decodingID(gs_fapi_server * server, gs_fapi_font * ff,
 }
 
 static gs_fapi_retcode
-gs_fapi_ufst_get_font_bbox(gs_fapi_server * server, gs_fapi_font * ff, int BBox[4])
+gs_fapi_ufst_get_font_bbox(gs_fapi_server * server, gs_fapi_font * ff, int BBox[4], int unitsPerEm[2])
 {
     fapi_ufst_server *r = If_to_I(server);
     SW16 VLCPower = 0;
@@ -1352,6 +1359,9 @@ gs_fapi_ufst_get_font_bbox(gs_fapi_server * server, gs_fapi_font * ff, int BBox[
     BBox[1] >>= VLCPower;
     BBox[2] >>= VLCPower;
     BBox[3] >>= VLCPower;
+
+    unitsPerEm[0] = unitsPerEm[1] = 1;
+
     return 0;
 }
 
@@ -1576,9 +1586,10 @@ set_metrics(fapi_ufst_server * r, gs_fapi_metrics * metrics,
 }
 
 static gs_fapi_retcode
-get_char(fapi_ufst_server * r, gs_fapi_font * ff, gs_fapi_char_ref * c,
-         gs_fapi_path * p, gs_fapi_metrics * metrics, UW16 format)
+get_char(gs_fapi_server *server, gs_fapi_font *ff, gs_fapi_char_ref *c,
+         gs_fapi_path *p, gs_fapi_metrics *metrics, UW16 format)
 {
+    fapi_ufst_server *r = If_to_I(server);
     UW16 code = 0, code2 = 0;
     UL32 cc = (UL32) c->char_codes[0];
     SL32 design_bbox[4];
@@ -1602,6 +1613,15 @@ get_char(fapi_ufst_server * r, gs_fapi_font * ff, gs_fapi_char_ref * c,
 
     design_escapement[0] = design_escapement[1] = 0;
 
+    /* NAFF! UFST requires that we recreate the MT font "object"
+     * each call (although, it should not!). So we want to defeat
+     * the optimisation that normally prevents that in gs_fapi_do_char()
+     * by blanking out the cached scale.
+     */
+    if(d->font_type == FC_FCO_TYPE) {
+        memset(&(server->face.ctm), 0x00, sizeof(server->face.ctm));
+    }
+
     if (ff->is_type1) {
         /* If a charstring in a Type 1 has been replaced with a PS procedure
          * get_glyph will return -1. We can then return char_code + 1 which
@@ -1813,8 +1833,14 @@ get_char(fapi_ufst_server * r, gs_fapi_font * ff, gs_fapi_char_ref * c,
             design_escapement[1] = 0;
         }
 
-        du_emx = pol->du_emx;
-        du_emy = pol->du_emy;
+        if (pol->du_emx > 0 && pol->du_emy > 0) {
+            du_emx = pol->du_emx;
+            du_emy = pol->du_emy;
+        }
+        else {
+            du_emx = pIFS->cs.du_emx;
+            du_emy = pIFS->cs.du_emy;
+        }
 
         design_bbox[0] = pol->left;
         design_bbox[1] = pol->bottom;
@@ -1922,13 +1948,13 @@ get_char(fapi_ufst_server * r, gs_fapi_font * ff, gs_fapi_char_ref * c,
 }
 
 static gs_fapi_retcode
-gs_fapi_ufst_get_char_outline_metrics(gs_fapi_server * server, gs_fapi_font * ff,
+gs_fapi_ufst_get_char_outline_metrics(gs_fapi_server *server, gs_fapi_font * ff,
                          gs_fapi_char_ref * c, gs_fapi_metrics * metrics)
 {
     fapi_ufst_server *r = If_to_I(server);
 
     gs_fapi_ufst_release_char_data_inline(r);
-    return get_char(r, ff, c, NULL, metrics, FC_CUBIC_TYPE);
+    return get_char(server, ff, c, NULL, metrics, FC_CUBIC_TYPE);
     /*  UFST cannot render enough metrics information without generating raster or outline.
        r->char_data keeps an outline after calling this function.
      */
@@ -1950,7 +1976,7 @@ gs_fapi_ufst_get_char_raster_metrics(gs_fapi_server * server, gs_fapi_font * ff,
     int code;
 
     gs_fapi_ufst_release_char_data_inline(r);
-    code = get_char(r, ff, c, NULL, metrics, FC_BITMAP_TYPE);
+    code = get_char(server, ff, c, NULL, metrics, FC_BITMAP_TYPE);
     if (code == ERR_bm_buff || code == ERR_bm_too_big)  /* Too big character ? */
         return (gs_error_VMerror);
     return code;
@@ -1968,7 +1994,7 @@ gs_fapi_ufst_get_char_width(gs_fapi_server * server, gs_fapi_font * ff,
 
     gs_fapi_ufst_release_char_data_inline(r);
     code =
-        get_char(r, ff, c, NULL, metrics,
+        get_char(server, ff, c, NULL, metrics,
                  server->use_outline ? FC_CUBIC_TYPE : FC_BITMAP_TYPE);
     if (code == ERR_bm_buff || code == ERR_bm_too_big)  /* Too big character ? */
         return (gs_error_VMerror);
@@ -2097,6 +2123,7 @@ static const gs_fapi_server ufstserver = {
     {0},
     0,
     false,
+    1,
     {1, 0, 0, 1, 0, 0},
     gs_fapi_ufst_ensure_open,
     gs_fapi_ufst_get_scaled_font,
