@@ -29,19 +29,26 @@
 #include "gstrans.h"
 #include "gxdownscale.h"
 
+#include "gdevkrnlsclass.h" /* 'standard' built in subclasses, currently First/Last Page and obejct filter */
+
 /*#define DEBUGGING_HACKS*/
 
 /* GC information */
 static
-ENUM_PTRS_WITH(device_printer_enum_ptrs, gx_device_printer *pdev)
+ENUM_PTRS_WITH(device_printer_enum_ptrs, gx_device_printer *pdev);
     if (PRINTER_IS_CLIST(pdev))
-        ENUM_PREFIX(st_device_clist, 0);
+        ENUM_PREFIX(st_device_clist, 2);
     else
-        ENUM_PREFIX(st_device_forward, 0);
+        ENUM_PREFIX(st_device_forward, 2);
+    break;
+case 0:ENUM_RETURN(gx_device_enum_ptr(pdev->parent));
+case 1:ENUM_RETURN(gx_device_enum_ptr(pdev->child));
 ENUM_PTRS_END
 static
 RELOC_PTRS_WITH(device_printer_reloc_ptrs, gx_device_printer *pdev)
 {
+    pdev->parent = gx_device_reloc_ptr(pdev->parent, gcst);
+    pdev->child = gx_device_reloc_ptr(pdev->child, gcst);
     if (PRINTER_IS_CLIST(pdev))
         RELOC_PREFIX(st_device_clist);
     else
@@ -76,17 +83,31 @@ static void prn_print_page_in_background(void *data);
 static void prn_finish_bg_print(gx_device_printer *ppdev);
 
 /* ------ Open/close ------ */
-
 /* Open a generic printer device. */
 /* Specific devices may wish to extend this. */
 int
 gdev_prn_open(gx_device * pdev)
 {
-    gx_device_printer * const ppdev = (gx_device_printer *)pdev;
+    gx_device_printer * ppdev;
     int code;
+    bool update_procs = false;
+
+    code = install_internal_subclass_devices(&pdev, &update_procs);
+    if (code < 0)
+        return code;
+
+    ppdev = (gx_device_printer *)pdev;
 
     ppdev->file = NULL;
     code = gdev_prn_allocate_memory(pdev, NULL, 0, 0);
+    if (update_procs) {
+        if (pdev->ObjectHandlerPushed) {
+            gx_copy_device_procs(&pdev->parent->procs, &pdev->procs, (gx_device_procs *)&gs_obj_filter_device.procs);
+            pdev = pdev->parent;
+        }
+        if (pdev->PageHandlerPushed)
+            gx_copy_device_procs(&pdev->parent->procs, &pdev->procs, (gx_device_procs *)&gs_flp_device.procs);
+    }
     if (code < 0)
         return code;
     if (ppdev->OpenOutputFile)
@@ -111,6 +132,24 @@ prn_finish_bg_print(gx_device_printer *ppdev)
         teardown_device_and_mem_for_thread(ppdev->bg_print.device,
                                            ppdev->bg_print.thread_id, true);
         ppdev->bg_print.device = NULL;
+        if (ppdev->bg_print.ocfile) {
+            closecode = ppdev->bg_print.oio_procs->fclose(ppdev->bg_print.ocfile, ppdev->bg_print.ocfname, true);
+            if (ppdev->bg_print.return_code == 0)
+               ppdev->bg_print.return_code = closecode;
+        }
+        if (ppdev->bg_print.ocfname) {
+            gs_free_object(ppdev->memory->non_gc_memory, ppdev->bg_print.ocfname, "prn_finish_bg_print(ocfname)");
+        }
+        if (ppdev->bg_print.obfile) {
+            closecode = ppdev->bg_print.oio_procs->fclose(ppdev->bg_print.obfile, ppdev->bg_print.obfname, true);
+            if (ppdev->bg_print.return_code == 0)
+               ppdev->bg_print.return_code = closecode;
+        }
+        if (ppdev->bg_print.obfname) {
+            gs_free_object(ppdev->memory->non_gc_memory, ppdev->bg_print.obfname, "prn_finish_bg_print(obfname)");
+        }
+        ppdev->bg_print.ocfile = ppdev->bg_print.obfile =
+          ppdev->bg_print.ocfname = ppdev->bg_print.obfname = NULL;
     }
 }
 /* Generic closing for the printer device. */
@@ -276,6 +315,29 @@ gdev_prn_tear_down(gx_device *pdev, byte **the_memory)
         ppdev->buffer_space = 0;
         was_command_list = true;
 
+        gs_free_object(pcldev->memory->non_gc_memory, pcldev->cache_chunk, "free tile cache for clist");
+        pcldev->cache_chunk = 0;
+        if (ppdev->bg_print.ocfile) {
+            (void)ppdev->bg_print.oio_procs->fclose(ppdev->bg_print.ocfile, ppdev->bg_print.ocfname, true);
+        }
+        if (ppdev->bg_print.obfile) {
+            (void)ppdev->bg_print.oio_procs->fclose(ppdev->bg_print.obfile, ppdev->bg_print.obfname, true);
+        }
+        ppdev->bg_print.ocfile = ppdev->bg_print.obfile = NULL;
+        if (ppdev->bg_print.ocfname) {
+            gs_free_object(ppdev->memory->non_gc_memory, ppdev->bg_print.ocfname, "gdev_prn_tear_down(ocfname)");
+        }
+        if (ppdev->bg_print.obfname) {
+            gs_free_object(ppdev->memory->non_gc_memory, ppdev->bg_print.obfname, "gdev_prn_tear_down(obfname)");
+        }
+        ppdev->bg_print.ocfname = ppdev->bg_print.obfname = NULL;
+
+        rc_decrement(pcldev->icc_cache_cl, "gdev_prn_tear_down");
+        pcldev->icc_cache_cl = NULL;
+
+        clist_free_icc_table(pcldev->icc_table, pcldev->memory);
+        pcldev->icc_table = NULL;
+
         /* If the clist is a reader clist, free any color_usage_array
          * memory used by same.
          */
@@ -417,6 +479,8 @@ gdev_prn_allocate(gx_device *pdev, gdev_prn_space_params *new_space_params,
             /* Buffer the image in a command list. */
             /* Release the buffer if we allocated it. */
             int code;
+            gx_device_printer * const ppdev = (gx_device_printer *)pdev;
+
             if (!reallocate) {
                 gs_free_object(buffer_memory, the_memory,
                                "printer buffer(open)");
@@ -426,6 +490,9 @@ gdev_prn_allocate(gx_device *pdev, gdev_prn_space_params *new_space_params,
                 ecode = gs_note_error(gs_error_VMerror);
                 continue;
             }
+            ppdev->bg_print.ocfname = ppdev->bg_print.obfname =
+                ppdev->bg_print.obfile = ppdev->bg_print.ocfile = NULL;
+
             code = gdev_prn_setup_as_command_list(pdev, buffer_memory,
                                                   &the_memory, &space_params,
                                                   !bufferSpace_is_default);
@@ -628,7 +695,7 @@ gdev_prn_get_param(gx_device *dev, char *Param, void *list)
     if (strcmp(Param, "pageneutralcolor") == 0) {
         return param_write_bool(plist, "pageneutralcolor", &pageneutralcolor);
     }
-    return gs_error_undefined;
+    return gx_default_get_param(dev, Param, list);
 }
 
 /* Get parameters.  Printer devices add several more parameters */
@@ -867,7 +934,7 @@ gdev_prn_put_params(gx_device * pdev, gs_param_list * plist)
     if (ppdev->bg_print_requested && !bg_print_requested) {
         prn_finish_bg_print(ppdev);
     }
-    
+
     ppdev->bg_print_requested = bg_print_requested;
     if (duplex_set >= 0) {
         ppdev->Duplex = duplex;
@@ -978,7 +1045,10 @@ gdev_prn_output_page_aux(gx_device * pdev, int num_copies, int flush, bool seeka
                 threads_enabled = 0;	/* and allow current page to try foreground */
             }
             /* Use 'while' instead of 'if' to avoid nesting */
-            while (ppdev->bg_print_requested && threads_enabled) {
+            while (ppdev->bg_print_requested && threads_enabled &&
+                   /* FIXME: Don't allow bg_print if multiple rendering threads are used.  */
+                   /* TEMPORARY WORK AROUND FOR BUG 695711 */
+                   ppdev->num_render_threads_requested == 0) {
                 gx_device *ndev;
                 gx_device_printer *npdev;
                 gx_device_clist_reader *crdev = (gx_device_clist_reader *)ppdev;
@@ -987,6 +1057,26 @@ gdev_prn_output_page_aux(gx_device * pdev, int num_copies, int flush, bool seeka
                     /* should not happen -- do foreground print */
                     break;
 
+                /* We need to hang onto references to these files, so we can ensure the main file data
+                 * gets freed with the correct allocator.
+                 */
+                ppdev->bg_print.ocfname =
+                     (char *)gs_alloc_bytes(ppdev->memory->non_gc_memory,
+                           strnlen(crdev->page_info.cfname, gp_file_name_sizeof - 1) + 1, "gdev_prn_output_page_aux(ocfname)");
+                ppdev->bg_print.obfname =
+                     (char *)gs_alloc_bytes(ppdev->memory->non_gc_memory,
+                           strnlen(crdev->page_info.bfname, gp_file_name_sizeof - 1) + 1,"gdev_prn_output_page_aux(ocfname)");
+
+                if (!ppdev->bg_print.ocfname || !ppdev->bg_print.obfname)
+                    break;
+
+                strncpy(ppdev->bg_print.ocfname, crdev->page_info.cfname, strnlen(crdev->page_info.cfname, gp_file_name_sizeof - 1) + 1);
+                strncpy(ppdev->bg_print.obfname, crdev->page_info.bfname, strnlen(crdev->page_info.bfname, gp_file_name_sizeof - 1) + 1);
+                ppdev->bg_print.obfile = crdev->page_info.bfile;
+                ppdev->bg_print.ocfile = crdev->page_info.cfile;
+                ppdev->bg_print.oio_procs = crdev->page_info.io_procs;
+                crdev->page_info.cfile = crdev->page_info.bfile = NULL;
+
                 if (ppdev->bg_print.sema == NULL)
                     if (((ppdev->bg_print.sema = gx_semaphore_alloc(ppdev->memory->non_gc_memory)) == NULL))
                         break;			/* couldn't create the semaphore */
@@ -1011,18 +1101,17 @@ gdev_prn_output_page_aux(gx_device * pdev, int num_copies, int flush, bool seeka
                 /* Page was succesfully started in bg_print mode */
                 print_foreground = 0;
                 /* Now we need to set up the next page so it will use new clist files */
-                /* Close (but don't delete) the files since the background printing now owns them */
-                if ((code = crdev->page_info.io_procs->fclose(crdev->page_info.cfile, crdev->page_info.cfname, false)) < 0 ||
-                    (code = crdev->page_info.io_procs->fclose(crdev->page_info.bfile, crdev->page_info.bfname, false)) < 0) {
-                    return_error(gs_error_unknownerror); /* shouldn't happen */
-                }
-                crdev->page_info.cfile = crdev->page_info.bfile = NULL;
                 if ((code = clist_open(pdev)) < 0) 	/* this should do it */
                     /* OOPS! can't proceed with the next page */
                     return code;	/* probably ioerror */
                 break;				/* exit the while loop */
             }
             if (print_foreground) {
+
+                 gs_free_object(ppdev->memory->non_gc_memory, ppdev->bg_print.ocfname, "gdev_prn_output_page_aux(ocfname)");
+                 gs_free_object(ppdev->memory->non_gc_memory, ppdev->bg_print.obfname, "gdev_prn_output_page_aux(obfname)");
+                 ppdev->bg_print.ocfname = ppdev->bg_print.obfname = NULL;
+
                 /* either bg_print was not requested or was not able to start */
                 if (ppdev->bg_print.sema != NULL && ppdev->bg_print.device != NULL) {
                     /* There was a problem. Teardown the device and its allocator, but */
@@ -1260,6 +1349,7 @@ gdev_prn_open_printer_seekable(gx_device *pdev, bool binary_mode,
         }
     }
     ppdev->file_is_new = true;
+
     return 0;
 }
 int
