@@ -33,6 +33,8 @@
 #include "gzcpath.h"
 #include "gxdevsop.h"
 
+#include "gdevkrnlsclass.h" /* 'standard' built in subclasses, currently First/Last Page and obejct filter */
+
 /* Structure descriptors */
 public_st_device_vector();
 public_st_vector_image_enum();
@@ -341,6 +343,11 @@ gdev_vector_open_file_options(gx_device_vector * vdev, uint strmbuf_size,
         (*dev_proc(vdev->bbox_device, open_device))
             ((gx_device *) vdev->bbox_device);
     }
+
+    code = install_internal_subclass_devices((gx_device **)&vdev, NULL);
+    if (code < 0)
+        return code;
+
     return 0;
 }
 
@@ -963,7 +970,7 @@ int gdev_vector_get_param(gx_device *dev, char *Param, void *list)
     if (strcmp(Param, "HighLevelDevice") == 0) {
         return param_write_bool(plist, "HighLevelDevice", &bool_true);
     }
-    return gs_error_undefined;
+    return gx_default_get_param(dev, Param, list);
 }
 
 /* Get parameters. */
@@ -999,7 +1006,6 @@ gdev_vector_put_params(gx_device * dev, gs_param_list * plist)
     gs_param_string ofns;
     bool open = dev->is_open, HighLevelDevice;
 
-
     code = param_read_bool(plist, (param_name = "HighLevelDevice"), &HighLevelDevice);
     if (code < 0)
         return code;
@@ -1054,6 +1060,7 @@ ofe:        param_signal_error(plist, param_name, ecode);
 
     if (ecode < 0)
         return ecode;
+
     {
         /* Don't let gx_default_put_params close the device. */
         dev->is_open = false;
@@ -1063,6 +1070,12 @@ ofe:        param_signal_error(plist, param_name, ecode);
     if (code < 0)
         return code;
 
+    if (dev->color_info.anti_alias.text_bits != 1 || dev->color_info.anti_alias.graphics_bits != 1) {
+        emprintf(dev->memory,
+            "\n\n  ERROR:\n    Can't set GraphicsAlphaBits or TextAlphaBits with a vector device.\n");
+        return gs_error_unregistered;
+    }
+
     if (ofns.data != 0) {
         /* If ofns.data is not NULL, then we have a different file name */
         memcpy(vdev->fname, ofns.data, ofns.size);
