@@ -181,11 +181,8 @@ main(int argc, char *argv[])
     define_int(f, "ARCH_ALIGN_INT_MOD", OFFSET_IN(si, i));
     define_int(f, "ARCH_ALIGN_LONG_MOD", OFFSET_IN(sl, l));
 
-#if defined (sparc)
-# ifndef __BIGGEST_ALIGNMENT__
-#  define __BIGGEST_ALIGNMENT__ 8
-# endif
-    define_int(f, "ARCH_ALIGN_PTR_MOD", __BIGGEST_ALIGNMENT__);
+#if defined (GS_MEMPTR_ALIGNMENT) && GS_MEMPTR_ALIGNMENT != 0
+    define_int(f, "ARCH_ALIGN_PTR_MOD", GS_MEMPTR_ALIGNMENT);
 #else
     define_int(f, "ARCH_ALIGN_PTR_MOD", OFFSET_IN(sp, p));
 #endif
