@@ -806,6 +806,27 @@ gp_fopen(const char *fname, const char *mode)
 #endif
 }
 
+int gp_stat(const char *path, struct _stat *buf)
+{
+#ifdef GS_NO_UTF8
+    return stat(path, buf);
+#else
+    int len = utf8_to_wchar(NULL, path);
+    wchar_t *uni;
+    int ret;
+
+    if (len <= 0)
+        return -1;
+    uni = malloc(len*sizeof(wchar_t));
+    if (uni == NULL)
+        return -1;
+    utf8_to_wchar(uni, path);
+    ret = _wstat(uni, buf);
+    free(uni);
+    return ret;
+#endif
+}
+
 /* test whether gp_fdup is supported on this platform  */
 int gp_can_share_fdesc(void)
 {
@@ -831,7 +852,7 @@ int gp_fpread(char *buf, uint count, int64_t offset, FILE *f)
 {
     OVERLAPPED overlapped;
     DWORD ret;
-    HANDLE hnd = _get_osfhandle(fileno(f));
+    HANDLE hnd = (HANDLE)_get_osfhandle(fileno(f));
 
     if (hnd == INVALID_HANDLE_VALUE)
         return -1;
@@ -851,7 +872,7 @@ int gp_fpwrite(char *buf, uint count, int64_t offset, FILE *f)
 {
     OVERLAPPED overlapped;
     DWORD ret;
-    HANDLE hnd = _get_osfhandle(fileno(f));
+    HANDLE hnd = (HANDLE)_get_osfhandle(fileno(f));
 
     if (hnd == INVALID_HANDLE_VALUE)
         return -1;
@@ -954,14 +975,14 @@ int gp_fseek_64(FILE *strm, gs_offset_t offset, int origin)
 
 bool gp_fseekable (FILE *f)
 {
-    struct stat s;
+    struct __stat64 s;
     int fno;
     
     fno = fileno(f);
     if (fno < 0)
         return(false);
     
-    if (fstat(fno, &s) < 0)
+    if (_fstat64(fno, &s) < 0)
         return(false);
 
     return((bool)S_ISREG(s.st_mode));
@@ -969,6 +990,9 @@ bool gp_fseekable (FILE *f)
 
 /* -------------------------  _snprintf -----------------------------*/
 
+#if defined(_MSC_VER) && _MSC_VER>=1900 /* VS 2014 and later have (finally) snprintf */
+#  define STDC99
+#else
 /* Microsoft Visual C++ 2005  doesn't properly define snprintf,
    which is defined in the C standard ISO/IEC 9899:1999 (E) */
 
@@ -986,3 +1010,4 @@ int snprintf(char *buffer, size_t count, const char *format, ...)
     } else
         return 0;
 }
+#endif
