@@ -151,6 +151,85 @@ gp_fopen(const char *fname, const char *mode)
     return fopen(fname, mode);
 }
 
+int gp_stat(const char *path, struct stat *buf)
+{
+    return stat(path, buf);
+}
+
+int gp_can_share_fdesc(void)
+{
+    return 1;
+}
+
+FILE *gp_open_scratch_file_rm(const gs_memory_t *mem,
+                              const char        *prefix,
+                                    char         fname[gp_file_name_sizeof],
+                              const char        *mode)
+{
+    FILE *f = gp_open_scratch_file_generic(mem, prefix, fname, mode, false);
+    /* Unlink file immediately to avoid it being left around if the program
+     * is killed. On this platform readers access temp files by cloning the
+     * FILE pointer and without accessing the file by name */
+    if (f)
+        unlink(fname);
+    return f;
+}
+
+FILE *gp_fdup(FILE *f, const char *mode)
+{
+    int fd = fileno(f);
+    if (fd < 0)
+        return NULL;
+    fd = dup(fd);
+    if (fd < 0)
+        return NULL;
+    return fdopen(fd, mode);
+}
+
+int gp_fpread(char *buf, uint count, int64_t offset, FILE *f)
+{
+#if defined(HAVE_PREAD_PWRITE) && HAVE_PREAD_PWRITE == 1
+    return pread(fileno(f), buf, count, offset);
+#else
+    int c;
+    int64_t os, curroff = gp_ftell_64(f);
+    if (curroff < 0) return curroff;
+    
+    os = gp_fseek_64(f, offset, 0);
+    if (os < 0) return os;
+    
+    c = fread(buf, 1, count, f);
+    if (c < 0) return c;
+    
+    os = gp_fseek_64(f, curroff, 0);
+    if (os < 0) return os;
+    
+    return c;
+#endif
+}
+
+int gp_fpwrite(char *buf, uint count, int64_t offset, FILE *f)
+{
+#if defined(HAVE_PREAD_PWRITE) && HAVE_PREAD_PWRITE == 1
+    return pwrite(fileno(f), buf, count, offset);
+#else
+    int c;
+    int64_t os, curroff = gp_ftell_64(f);
+    if (curroff < 0) return curroff;
+    
+    os = gp_fseek_64(f, offset, 0);
+    if (os < 0) return os;
+    
+    c = fwrite(buf, 1, count, f);
+    if (c < 0) return c;
+    
+    os = gp_fseek_64(f, curroff, 0);
+    if (os < 0) return os;
+    
+    return c;
+#endif
+}
+
 /* Set a file into binary or text mode. */
 int
 gp_setmode_binary(FILE * pfile, bool mode)
