@@ -68,11 +68,11 @@ LOOKUP_ENTRY(cie_cached_value vin, const gx_cie_vector_cache *pcache)
  * the extra level of procedure.
  */
 #ifdef DEBUG
-#  define GX_CIE_REMAP_FINISH(vec3, pconc, pis, pcs)\
-    gx_cie_remap_finish(vec3, pconc, pis, pcs)
+#  define GX_CIE_REMAP_FINISH(vec3, pconc, cie_xyz, pis, pcs)\
+    gx_cie_remap_finish(vec3, pconc, cie_xyz, pis, pcs)
 #else
-#  define GX_CIE_REMAP_FINISH(vec3, pconc, pis, pcs)\
-    ((pis)->cie_joint_caches->remap_finish(vec3, pconc, pis, pcs))
+#  define GX_CIE_REMAP_FINISH(vec3, pconc, cie_xyz, pis, pcs)\
+    ((pis)->cie_joint_caches->remap_finish(vec3, pconc, cie_xyz, pis, pcs))
 #endif
 
 /* Forward references */
@@ -109,39 +109,39 @@ check_range(gs_range *ranges, int num_colorants)
         if (ranges[k].rmin != 0) return false;
         if (ranges[k].rmax != 1) return false;
     }
-    return(true);
+    return true;
 }
 
 /* Returns false if range is not 0 1 */
 bool
-check_cie_range( const gs_color_space * pcs )
+check_cie_range(const gs_color_space * pcs)
 {
-    switch(gs_color_space_get_index(pcs)){
+    switch(gs_color_space_get_index(pcs)) {
         case gs_color_space_index_CIEDEFG:
-            return(check_range(&(pcs->params.defg->RangeDEFG.ranges[0]), 4));
+            return check_range(&(pcs->params.defg->RangeDEFG.ranges[0]), 4) ;
         case gs_color_space_index_CIEDEF:
-            return(check_range(&(pcs->params.def->RangeDEF.ranges[0]), 3));
+            return check_range(&(pcs->params.def->RangeDEF.ranges[0]), 3);
         case gs_color_space_index_CIEABC:
-            return(check_range(&(pcs->params.abc->RangeABC.ranges[0]), 3));
+            return check_range(&(pcs->params.abc->RangeABC.ranges[0]), 3);
         case gs_color_space_index_CIEA:
-            return(check_range(&(pcs->params.a->RangeA), 1));
+            return check_range(&(pcs->params.a->RangeA), 1);
         default:
             return true;
     }
 }
 
 gs_range*
-get_cie_range( const gs_color_space * pcs )
+get_cie_range(const gs_color_space * pcs)
 {
-    switch(gs_color_space_get_index(pcs)){
+    switch(gs_color_space_get_index(pcs)) {
         case gs_color_space_index_CIEDEFG:
-            return(&(pcs->params.defg->RangeDEFG.ranges[0]));
+            return &(pcs->params.defg->RangeDEFG.ranges[0]);
         case gs_color_space_index_CIEDEF:
-            return(&(pcs->params.def->RangeDEF.ranges[0]));
+            return &(pcs->params.def->RangeDEF.ranges[0]);
         case gs_color_space_index_CIEABC:
-            return(&(pcs->params.abc->RangeABC.ranges[0]));
+            return &(pcs->params.abc->RangeABC.ranges[0]);
         case gs_color_space_index_CIEA:
-            return(&(pcs->params.a->RangeA));
+            return &(pcs->params.a->RangeA);
         default:
             return NULL;
     }
@@ -154,12 +154,48 @@ rescale_input_color(gs_range *ranges, int num_colorants,
     int k;
 
     for (k = 0; k < num_colorants; k++) {
-        des->paint.values[k] =
-            (src->paint.values[k]-ranges[k].rmin)/
-            (ranges[k].rmax-ranges[k].rmin);
+        des->paint.values[k] = (src->paint.values[k] - ranges[k].rmin) /
+                               (ranges[k].rmax - ranges[k].rmin);
     }
 }
 
+/* Returns true if rescale was done */
+bool
+rescale_cie_colors(const gs_color_space * pcs, gs_client_color *cc)
+{
+    int num, k;
+    gs_range *ranges;
+
+    if (!check_cie_range(pcs)) {
+        switch(gs_color_space_get_index(pcs)) {
+            case gs_color_space_index_CIEDEFG:
+                num = 4;
+                ranges = &(pcs->params.defg->RangeDEFG.ranges[0]);
+                break;
+            case gs_color_space_index_CIEDEF:
+                num = 3;
+                ranges = &(pcs->params.def->RangeDEF.ranges[0]);
+                break;
+            case gs_color_space_index_CIEABC:
+                num = 3;
+                ranges = &(pcs->params.abc->RangeABC.ranges[0]);
+                break;
+            case gs_color_space_index_CIEA:
+                num = 1;
+                ranges = &(pcs->params.a->RangeA);
+                break;
+            default:
+                return false;
+        }
+        for (k = 0; k < num; k++) {
+            cc->paint.values[k] = (cc->paint.values[k] - ranges[k].rmin) /
+                                    (ranges[k].rmax - ranges[k].rmin);
+        }
+        return true;
+    }
+    return false;
+}
+
 /*
  * Test whether a CIE rendering has been defined; ensure that the joint
  * caches are loaded.  Note that the procedure may return 1 if no rendering
@@ -205,18 +241,24 @@ gx_ciedefg_to_icc(gs_color_space **ppcs_icc, gs_color_space *pcs, gs_memory_t *m
                "[icc] Creating ICC profile from defg object");  
     /* build the ICC color space object */
     code = gs_cspace_build_ICC(ppcs_icc, NULL, memory->stable_memory);
+    if (code < 0)
+        gs_rethrow(code, "Failed to build ICC color space");
     /* record the cie alt space as the icc alternative color space */
     (*ppcs_icc)->base_space = palt_cs;
     rc_increment_cs(palt_cs);
     (*ppcs_icc)->cmm_icc_profile_data = gsicc_profile_new(NULL, memory, NULL, 0);
+    if ((*ppcs_icc)->cmm_icc_profile_data == NULL)
+        gs_throw(gs_error_VMerror, "Failed to create ICC profile");
     code = gsicc_create_fromdefg(pcs, &((*ppcs_icc)->cmm_icc_profile_data->buffer),
                     &((*ppcs_icc)->cmm_icc_profile_data->buffer_size), memory,
                     abc_caches, lmn_caches, defg_caches);
+    if (code < 0)
+        gs_rethrow(code, "Failed to create ICC profile from CIEDEFG");
     gsicc_init_profile_info((*ppcs_icc)->cmm_icc_profile_data);
     (*ppcs_icc)->cmm_icc_profile_data->default_match = CIE_DEFG;
     pcs->icc_equivalent = *ppcs_icc;
     pcs->icc_equivalent->cmm_icc_profile_data->data_cs = gsCMYK;
-    return(0);
+    return 0;
 }
 
 int
@@ -237,13 +279,15 @@ gx_remap_CIEDEFG(const gs_client_color * pc, const gs_color_space * pcs_in,
        will finish that process now. */
     if (pcs->icc_equivalent == NULL) {
         code = gx_ciedefg_to_icc(&pcs_icc, pcs, pis->memory->stable_memory);
+        if (code < 0)
+            gs_rethrow(code, "Failed to create ICC profile from CIEDEFG");
     } else {
         pcs_icc = pcs->icc_equivalent;
     }
     /* Rescale the input based upon the input range since profile is
        created to remap this range from 0 to 1 */
     if (check_range(&(pcs->params.defg->RangeDEFG.ranges[0]), 4)) {
-        return((pcs_icc->type->remap_color)(pc,pcs_icc,pdc,pis,dev,select));
+        return (pcs_icc->type->remap_color)(pc,pcs_icc,pdc,pis,dev,select);
     }
     /* Do the rescale from 0 to 1 */
     rescale_input_color(&(pcs->params.defg->RangeDEFG.ranges[0]), 4, pc, &scale_pc);
@@ -253,7 +297,7 @@ gx_remap_CIEDEFG(const gs_client_color * pc, const gs_color_space * pcs_in,
     for (i = 0; i < 4; i++)
         pdc->ccolor.paint.values[i] = pc->paint.values[i];
     pdc->ccolor_valid = true;
-    return(code);
+    return code;
 }
 
 /* Render a CIEBasedDEFG color. */
@@ -274,24 +318,26 @@ gx_concretize_CIEDEFG(const gs_client_color * pc, const gs_color_space * pcs_in,
        will finish that process now. */
     if (pcs->icc_equivalent == NULL) {
         code = gx_ciedefg_to_icc(&pcs_icc, pcs, pis->memory->stable_memory);
+        if (code < 0)
+            gs_rethrow(code, "Failed to create ICC profile from CIEDEFG");
     } else {
         pcs_icc = pcs->icc_equivalent;
     }
     /* Rescale the input based upon the input range since profile is
        created to remap this range from 0 to 1 */
     if (check_range(&(pcs->params.defg->RangeDEFG.ranges[0]), 4)) {
-        return((pcs_icc->type->concretize_color)(pc, pcs_icc, pconc, pis, dev));
+        return (pcs_icc->type->concretize_color)(pc, pcs_icc, pconc, pis, dev);
     }
     /* Do the rescale from 0 to 1 */
     rescale_input_color(&(pcs->params.defg->RangeDEFG.ranges[0]), 4, pc, &scale_pc);
     /* Now the icc remap */
-    return((pcs_icc->type->concretize_color)(pc, pcs_icc, pconc, pis, dev));
+    return (pcs_icc->type->concretize_color)(pc, pcs_icc, pconc, pis, dev);
 }
 
 /* Used for when we have to mash entire transform to CIEXYZ */
 int
 gx_psconcretize_CIEA(const gs_client_color * pc, const gs_color_space * pcs,
-                      frac * pconc, const gs_imager_state * pis)
+                      frac * pconc, float * cie_xyz, const gs_imager_state * pis)
 {
     const gs_cie_a *pcie = pcs->params.a;
     cie_cached_value a = float2cie_cached(pc->paint.values[0]);
@@ -310,14 +356,14 @@ gx_psconcretize_CIEA(const gs_client_color * pc, const gs_color_space * pcs,
         vlmn = *LOOKUP_ENTRY(a, &pcie->caches.DecodeA);
     else
         vlmn.u = vlmn.v = vlmn.w = a;
-    GX_CIE_REMAP_FINISH(vlmn, pconc, pis, pcs);
+    GX_CIE_REMAP_FINISH(vlmn, pconc, cie_xyz, pis, pcs);
     return 0;
 }
 
 /* Used for when we have to mash entire transform to CIEXYZ */
 int
 gx_psconcretize_CIEABC(const gs_client_color * pc, const gs_color_space * pcs,
-                      frac * pconc, const gs_imager_state * pis)
+                      frac * pconc, float * cie_xyz, const gs_imager_state * pis)
 {
     const gs_cie_abc *pcie = pcs->params.abc;
     cie_cached_vector3 vec3;
@@ -338,14 +384,14 @@ gx_psconcretize_CIEABC(const gs_client_color * pc, const gs_color_space * pcs,
     if (!pis->cie_joint_caches->skipDecodeABC)
         cie_lookup_map3(&vec3 /* ABC => LMN */, &pcie->caches.DecodeABC,
                         "Decode/MatrixABC");
-    GX_CIE_REMAP_FINISH(vec3, pconc, pis, pcs);
+    GX_CIE_REMAP_FINISH(vec3, pconc, cie_xyz, pis, pcs);
     return 0;
 }
 
 /* Used for when we have to mash entire transform to CIEXYZ */
 int
 gx_psconcretize_CIEDEFG(const gs_client_color * pc, const gs_color_space * pcs,
-                      frac * pconc, const gs_imager_state * pis)
+                      frac * pconc, float * cie_xyz, const gs_imager_state * pis)
 {
     const gs_cie_defg *pcie = pcs->params.defg;
     int i;
@@ -400,14 +446,14 @@ gx_psconcretize_CIEDEFG(const gs_client_color * pc, const gs_color_space * pcs,
     if (!pis->cie_joint_caches->skipDecodeABC)
         cie_lookup_map3(&vec3 /* ABC => LMN */, &pcie->caches.DecodeABC,
                         "Decode/MatrixABC");
-    GX_CIE_REMAP_FINISH(vec3, pconc, pis, pcs);
+    GX_CIE_REMAP_FINISH(vec3, pconc, cie_xyz, pis, pcs);
     return 0;
 }
 
 /* Render a CIEBasedDEF color. */
 int
 gx_psconcretize_CIEDEF(const gs_client_color * pc, const gs_color_space * pcs,
-                     frac * pconc, const gs_imager_state * pis)
+                     frac * pconc, float * cie_xyz, const gs_imager_state * pis)
 {
     const gs_cie_def *pcie = pcs->params.def;
     int i;
@@ -458,7 +504,7 @@ gx_psconcretize_CIEDEF(const gs_client_color * pc, const gs_color_space * pcs,
     if (!pis->cie_joint_caches->skipDecodeABC)
         cie_lookup_map3(&vec3 /* ABC => LMN */, &pcie->caches.DecodeABC,
                         "Decode/MatrixABC");
-    GX_CIE_REMAP_FINISH(vec3, pconc, pis, pcs);
+    GX_CIE_REMAP_FINISH(vec3, pconc, cie_xyz, pis, pcs);
     return 0;
 }
 #undef SCALE_TO_RANGE
@@ -467,7 +513,7 @@ gx_psconcretize_CIEDEF(const gs_client_color * pc, const gs_color_space * pcs,
 static int
 gx_ciedef_to_icc(gs_color_space **ppcs_icc, gs_color_space *pcs, gs_memory_t *memory)
 {
-    int code = 0;
+    int code;
     gs_color_space *palt_cs = pcs->base_space;
     gx_cie_vector_cache *abc_caches = &(pcs->params.abc->caches.DecodeABC.caches[0]);
     gx_cie_scalar_cache    *lmn_caches = &(pcs->params.abc->common.caches.DecodeLMN[0]);
@@ -476,19 +522,25 @@ gx_ciedef_to_icc(gs_color_space **ppcs_icc, gs_color_space *pcs, gs_memory_t *me
     if_debug0(gs_debug_flag_icc,"[icc] Creating ICC profile from def object");  
     /* build the ICC color space object */
     code = gs_cspace_build_ICC(ppcs_icc, NULL, memory->stable_memory);
+    if (code < 0)
+        gs_rethrow(code, "Failed to build ICC color space");
     /* record the cie alt space as the icc alternative color space */
     (*ppcs_icc)->base_space = palt_cs;
     rc_increment_cs(palt_cs);
     (*ppcs_icc)->cmm_icc_profile_data = gsicc_profile_new(NULL, memory, NULL, 0);
+    if ((*ppcs_icc)->cmm_icc_profile_data == NULL)
+        gs_throw(gs_error_VMerror, "Failed to create ICC profile");
     code = gsicc_create_fromdef(pcs, &((*ppcs_icc)->cmm_icc_profile_data->buffer),
                     &((*ppcs_icc)->cmm_icc_profile_data->buffer_size), memory,
                     abc_caches, lmn_caches, def_caches);
+    if (code < 0)
+        gs_rethrow(code, "Failed to build ICC profile from CIEDEF");
     gsicc_init_profile_info((*ppcs_icc)->cmm_icc_profile_data);
     (*ppcs_icc)->cmm_icc_profile_data->default_match = CIE_DEF;
     /* Assign to the icc_equivalent member variable */
     pcs->icc_equivalent = *ppcs_icc;
     pcs->icc_equivalent->cmm_icc_profile_data->data_cs = gsUNDEFINED;
-    return(0);
+    return 0;
     }
 
 int
@@ -508,14 +560,16 @@ gx_remap_CIEDEF(const gs_client_color * pc, const gs_color_space * pcs_in,
        the conversion of the DEF space to an ICC type.  We
        will finish that process now. */
     if (pcs->icc_equivalent == NULL) {
-        gx_ciedef_to_icc(&pcs_icc, pcs, pis->memory->stable_memory);
+        code = gx_ciedef_to_icc(&pcs_icc, pcs, pis->memory->stable_memory);
+        if (code < 0)
+            gs_rethrow(code, "Failed to build ICC profile from CIEDEF");
     } else {
         pcs_icc = pcs->icc_equivalent;
     }
     /* Rescale the input based upon the input range since profile is
        created to remap this range from 0 to 1 */
     if (check_range(&(pcs->params.def->RangeDEF.ranges[0]), 3)) {
-        return((pcs_icc->type->remap_color)(pc,pcs_icc,pdc,pis,dev,select));
+        return (pcs_icc->type->remap_color)(pc,pcs_icc,pdc,pis,dev,select);
     }
     /* Do the rescale from 0 to 1 */
     rescale_input_color(&(pcs->params.def->RangeDEF.ranges[0]), 3, pc, &scale_pc);
@@ -525,7 +579,7 @@ gx_remap_CIEDEF(const gs_client_color * pc, const gs_color_space * pcs_in,
     for (i = 0; i < 3; i++)
         pdc->ccolor.paint.values[i] = pc->paint.values[i];
     pdc->ccolor_valid = true;
-    return(code);
+    return code;
 }
 
 /* Render a CIEBasedDEF color. */
@@ -533,7 +587,7 @@ int
 gx_concretize_CIEDEF(const gs_client_color * pc, const gs_color_space * pcs_in,
                      frac * pconc, const gs_imager_state * pis, gx_device *dev)
 {
-    int code;
+    int code = 0;
     gs_color_space *pcs_icc;
     gs_client_color scale_pc;
     gs_color_space *pcs = (gs_color_space *) pcs_in;
@@ -546,18 +600,20 @@ gx_concretize_CIEDEF(const gs_client_color * pc, const gs_color_space * pcs_in,
        will finish that process now. */
     if (pcs->icc_equivalent == NULL) {
         code = gx_ciedef_to_icc(&pcs_icc, pcs, pis->memory->stable_memory);
+        if (code < 0)
+            gs_rethrow(code, "Failed to build ICC profile from CIEDEF");
     } else {
         pcs_icc = pcs->icc_equivalent;
     }
     /* Rescale the input based upon the input range since profile is
        created to remap this range from 0 to 1 */
     if (check_range(&(pcs->params.def->RangeDEF.ranges[0]), 3)) {
-        return((pcs_icc->type->concretize_color)(pc, pcs_icc, pconc, pis, dev));
+        return (pcs_icc->type->concretize_color)(pc, pcs_icc, pconc, pis, dev);
 }
     /* Do the rescale from 0 to 1 */
     rescale_input_color(&(pcs->params.def->RangeDEF.ranges[0]), 3, pc, &scale_pc);
     /* Now the icc remap */
-    return((pcs_icc->type->concretize_color)(&scale_pc, pcs_icc, pconc, pis, dev));
+    return (pcs_icc->type->concretize_color)(&scale_pc, pcs_icc, pconc, pis, dev);
 }
 #undef SCALE_TO_RANGE
 
@@ -566,7 +622,7 @@ static int
 gx_cieabc_to_icc(gs_color_space **ppcs_icc, gs_color_space *pcs, bool *islab,
                  gs_memory_t *memory)
 {
-    int code = 0;
+    int code;
     gs_color_space *palt_cs = pcs->base_space;
     gx_cie_vector_cache *abc_caches = &(pcs->params.abc->caches.DecodeABC.caches[0]);
     gx_cie_scalar_cache *lmn_caches = &(pcs->params.abc->common.caches.DecodeLMN[0]);
@@ -574,19 +630,25 @@ gx_cieabc_to_icc(gs_color_space **ppcs_icc, gs_color_space *pcs, bool *islab,
     if_debug0m(gs_debug_flag_icc, memory, "[icc] Creating ICC profile from abc object");  
     /* build the ICC color space object */
     code = gs_cspace_build_ICC(ppcs_icc, NULL, memory);
+    if (code < 0)
+        gs_rethrow(code, "Failed to create ICC profile");
     /* record the cie alt space as the icc alternative color space */
     (*ppcs_icc)->base_space = palt_cs;
     rc_increment_cs(palt_cs);
     (*ppcs_icc)->cmm_icc_profile_data = gsicc_profile_new(NULL, memory, NULL, 0);
+    if ((*ppcs_icc)->cmm_icc_profile_data == NULL)
+        gs_throw(gs_error_VMerror, "Failed to create ICC profile");
     code = gsicc_create_fromabc(pcs, &((*ppcs_icc)->cmm_icc_profile_data->buffer),
                     &((*ppcs_icc)->cmm_icc_profile_data->buffer_size), memory,
                     abc_caches, lmn_caches, islab);
+    if (code < 0)
+        gs_rethrow(code, "Failed to build ICC profile from CIEABC");
     gsicc_init_profile_info((*ppcs_icc)->cmm_icc_profile_data);
     (*ppcs_icc)->cmm_icc_profile_data->default_match = CIE_ABC;
     /* Assign to the icc_equivalent member variable */
     pcs->icc_equivalent = *ppcs_icc;
     pcs->icc_equivalent->cmm_icc_profile_data->data_cs = gsRGB;
-    return(0);
+    return 0;
     }
 
 /* Render a CIEBasedABC color. */
@@ -600,10 +662,9 @@ gx_remap_CIEABC(const gs_client_color * pc, const gs_color_space * pcs_in,
     gs_color_space *pcs_icc;
     gs_client_color scale_pc;
     bool islab;
-    int i,code;
+    int i, code;
     gs_color_space *pcs = (gs_color_space *) pcs_in;
 
-
     if_debug3m('c', pis->memory, "[c]remap CIEABC [%g %g %g]\n",
                pc->paint.values[0], pc->paint.values[1],
                pc->paint.values[2]);
@@ -611,14 +672,16 @@ gx_remap_CIEABC(const gs_client_color * pc, const gs_color_space * pcs_in,
        the conversion of the ABC space to an ICC type.  We
        will finish that process now. */
     if (pcs->icc_equivalent == NULL) {
-        gx_cieabc_to_icc(&pcs_icc, pcs, &islab, pis->memory->stable_memory);
+        code = gx_cieabc_to_icc(&pcs_icc, pcs, &islab, pis->memory->stable_memory);
+        if (code < 0)
+            gs_rethrow(code, "Failed to create ICC profile from CIEABC");
     } else {
         pcs_icc = pcs->icc_equivalent;
-        }
+    }
     /* Rescale the input based upon the input range since profile is
        created to remap this range from 0 to 1 */
     if (check_range(&(pcs->params.abc->RangeABC.ranges[0]), 3)) {
-        return((pcs_icc->type->remap_color)(pc,pcs_icc,pdc,pis,dev,select));
+        return (pcs_icc->type->remap_color)(pc,pcs_icc,pdc,pis,dev,select);
     }
     /* Do the rescale from 0 to 1 */
     rescale_input_color(&(pcs->params.abc->RangeABC.ranges[0]), 3, pc, &scale_pc);
@@ -629,7 +692,7 @@ gx_remap_CIEABC(const gs_client_color * pc, const gs_color_space * pcs_in,
         pdc->ccolor.paint.values[i] = pc->paint.values[i];
     pdc->ccolor_valid = true;
     /* Now the icc remap */
-    return(code);
+    return code;
 }
 
 int
@@ -640,6 +703,7 @@ gx_concretize_CIEABC(const gs_client_color * pc, const gs_color_space * pcs_in,
     gs_client_color scale_pc;
     bool islab;
     gs_color_space *pcs = (gs_color_space *) pcs_in;
+    int code = 0;
 
     if_debug3m('c', pis->memory, "[c]concretize CIEABC [%g %g %g]\n",
                pc->paint.values[0], pc->paint.values[1],
@@ -648,26 +712,28 @@ gx_concretize_CIEABC(const gs_client_color * pc, const gs_color_space * pcs_in,
        the conversion of the ABC space to an ICC type.  We
        will finish that process now. */
     if (pcs->icc_equivalent == NULL) {
-        gx_cieabc_to_icc(&pcs_icc, pcs, &islab, pis->memory->stable_memory);
+        code = gx_cieabc_to_icc(&pcs_icc, pcs, &islab, pis->memory->stable_memory);
+        if (code < 0)
+            gs_rethrow(code, "Failed to create ICC profile from CIEABC");
     } else {
         pcs_icc = pcs->icc_equivalent;
     }
     /* Rescale the input based upon the input range since profile is
        created to remap this range from 0 to 1 */
     if (check_range(&(pcs->params.abc->RangeABC.ranges[0]), 3)) {
-        return((pcs_icc->type->concretize_color)(pc, pcs_icc, pconc, pis, dev));
+        return (pcs_icc->type->concretize_color)(pc, pcs_icc, pconc, pis, dev);
     }
     /* Do the rescale from 0 to 1 */
     rescale_input_color(&(pcs->params.abc->RangeABC.ranges[0]), 3, pc, &scale_pc);
     /* Now the icc remap */
-    return((pcs_icc->type->concretize_color)(&scale_pc, pcs_icc, pconc, pis, dev));
+    return (pcs_icc->type->concretize_color)(&scale_pc, pcs_icc, pconc, pis, dev);
 }
 
 /* Common code shared between remap and concretize */
 static int
 gx_ciea_to_icc(gs_color_space **ppcs_icc, gs_color_space *pcs, gs_memory_t *memory)
 {
-    int code = 0;
+    int code;
     gs_color_space *palt_cs = pcs->base_space;
     gx_cie_vector_cache *a_cache = &(pcs->params.a->caches.DecodeA);
     gx_cie_scalar_cache    *lmn_caches = &(pcs->params.a->common.caches.DecodeLMN[0]);
@@ -676,19 +742,25 @@ gx_ciea_to_icc(gs_color_space **ppcs_icc, gs_color_space *pcs, gs_memory_t *memo
                "[icc] Creating ICC profile from CIEA object");  
     /* build the ICC color space object */
     code = gs_cspace_build_ICC(ppcs_icc, NULL, memory);
+    if (code < 0)
+        gs_rethrow(code, "Failed to create ICC profile");
     /* record the cie alt space as the icc alternative color space */
     (*ppcs_icc)->base_space = palt_cs;
     rc_increment_cs(palt_cs);
     (*ppcs_icc)->cmm_icc_profile_data = gsicc_profile_new(NULL, memory, NULL, 0);
+    if ((*ppcs_icc)->cmm_icc_profile_data == NULL)
+        gs_throw(gs_error_VMerror, "Failed to create ICC profile");
     code = gsicc_create_froma(pcs, &((*ppcs_icc)->cmm_icc_profile_data->buffer),
                     &((*ppcs_icc)->cmm_icc_profile_data->buffer_size), memory,
                     a_cache, lmn_caches);
+    if (code < 0)
+        gs_rethrow(code, "Failed to create ICC profile from CIEA");
     gsicc_init_profile_info((*ppcs_icc)->cmm_icc_profile_data);
     (*ppcs_icc)->cmm_icc_profile_data->default_match = CIE_A;
     /* Assign to the icc_equivalent member variable */
     pcs->icc_equivalent = *ppcs_icc;
     pcs->icc_equivalent->cmm_icc_profile_data->data_cs = gsGRAY;
-    return(code);
+    return 0;
 }
 
 int
@@ -707,6 +779,8 @@ gx_remap_CIEA(const gs_client_color * pc, const gs_color_space * pcs_in,
        will finish that process now. */
     if (pcs->icc_equivalent == NULL) {
         code = gx_ciea_to_icc(&pcs_icc, pcs, pis->memory->stable_memory);
+        if (code < 0)
+            gs_rethrow(code, "Failed to create ICC profile from CIEA");
     } else {
         /* Once the ICC color space is set, we should be doing all the remaps through the ICC equivalent */
         pcs_icc = pcs->icc_equivalent;
@@ -714,7 +788,7 @@ gx_remap_CIEA(const gs_client_color * pc, const gs_color_space * pcs_in,
     /* Rescale the input based upon the input range since profile is
        created to remap this range from 0 to 1 */
     if (check_range(&(pcs->params.a->RangeA), 1)) {
-        return((pcs_icc->type->remap_color)(pc,pcs_icc,pdc,pis,dev,select));
+        return (pcs_icc->type->remap_color)(pc,pcs_icc,pdc,pis,dev,select);
     }
     /* Do the rescale from 0 to 1 */
     rescale_input_color(&(pcs->params.a->RangeA), 1, pc, &scale_pc);
@@ -723,7 +797,7 @@ gx_remap_CIEA(const gs_client_color * pc, const gs_color_space * pcs_in,
     /* Save unscaled data for high level device (e.g. pdfwrite) */
     pdc->ccolor.paint.values[0] = pc->paint.values[0];
     pdc->ccolor_valid = true;
-    return(code);
+    return code;
 }
 
 /* Render a CIEBasedA color. */
@@ -731,7 +805,7 @@ int
 gx_concretize_CIEA(const gs_client_color * pc, const gs_color_space * pcs_in,
                    frac * pconc, const gs_imager_state * pis, gx_device *dev)
 {
-    int code;
+    int code = 0;
     gs_color_space *pcs_icc;
     gs_client_color scale_pc;
     gs_color_space *pcs = (gs_color_space *) pcs_in;
@@ -742,6 +816,8 @@ gx_concretize_CIEA(const gs_client_color * pc, const gs_color_space * pcs_in,
        will finish that process now. */
     if (pcs->icc_equivalent == NULL) {
         code = gx_ciea_to_icc(&pcs_icc, pcs, pis->memory->stable_memory);
+        if (code < 0)
+            gs_rethrow(code, "Failed to create ICC profile from CIEA");
     } else {
         /* Once the ICC color space is set, we should be doing all the remaps through the ICC equivalent */
         pcs_icc = pcs->icc_equivalent;
@@ -749,12 +825,12 @@ gx_concretize_CIEA(const gs_client_color * pc, const gs_color_space * pcs_in,
     /* Rescale the input based upon the input range since profile is
        created to remap this range from 0 to 1 */
     if (check_range(&(pcs->params.a->RangeA), 1)) {
-        return((pcs_icc->type->concretize_color)(pc, pcs_icc, pconc, pis, dev));
+        return (pcs_icc->type->concretize_color)(pc, pcs_icc, pconc, pis, dev);
     }
     /* Do the rescale from 0 to 1 */
     rescale_input_color(&(pcs->params.a->RangeA), 1, pc, &scale_pc);
     /* Now the icc remap */
-    return((pcs_icc->type->concretize_color)(&scale_pc, pcs_icc, pconc, pis, dev));
+    return (pcs_icc->type->concretize_color)(&scale_pc, pcs_icc, pconc, pis, dev);
 }
 
 /* Call for cases where the equivalent icc color space needs to be set */
@@ -762,14 +838,14 @@ int
 gs_colorspace_set_icc_equivalent(gs_color_space *pcs, bool *islab,
                                  gs_memory_t *memory)
 {
-     gs_color_space_index color_space_index = gs_color_space_get_index(pcs);
-     gs_color_space *picc_cs;
+    gs_color_space_index color_space_index = gs_color_space_get_index(pcs);
+    gs_color_space *picc_cs;
 
-     *islab = false;  /* For non CIEABC cases */
-     if (pcs->icc_equivalent != NULL || !gs_color_space_is_PSCIE(pcs)) {
-         return(0);
-}
-     switch( color_space_index ) {
+    *islab = false;  /* For non CIEABC cases */
+    if (pcs->icc_equivalent != NULL || !gs_color_space_is_PSCIE(pcs))
+        return 0;
+
+    switch( color_space_index ) {
        case gs_color_space_index_CIEDEFG:
             gx_ciedefg_to_icc(&picc_cs, pcs, memory->stable_memory);
             break;
@@ -785,24 +861,24 @@ gs_colorspace_set_icc_equivalent(gs_color_space *pcs, bool *islab,
         default:
              /* do nothing.  Sould never happen */
              break;
-     }
-    return(0);
+    }
+    return 0;
 }
 
 /* Call the remap_finish procedure in the joint_caches structure. */
 int
-gx_cie_remap_finish(cie_cached_vector3 vec3, frac * pconc,
+gx_cie_remap_finish(cie_cached_vector3 vec3, frac * pconc, float *cie_xyz,
                     const gs_imager_state * pis,
                     const gs_color_space *pcs)
 {
-    return pis->cie_joint_caches->remap_finish(vec3, pconc, pis, pcs);
+    return pis->cie_joint_caches->remap_finish(vec3, pconc, cie_xyz, pis, pcs);
 }
 
 /* Finish remapping a CIEBased color. */
 /* Return 3 if RGB, 4 if CMYK. */
 /* this procedure is exported for the benefit of gsicc.c */
 int
-gx_cie_real_remap_finish(cie_cached_vector3 vec3, frac * pconc,
+gx_cie_real_remap_finish(cie_cached_vector3 vec3, frac * pconc, float * xyz,
                          const gs_imager_state * pis,
                          const gs_color_space *pcs)
 {
@@ -811,6 +887,7 @@ gx_cie_real_remap_finish(cie_cached_vector3 vec3, frac * pconc,
     const gs_const_string *table = pcrd->RenderTable.lookup.table;
     int tabc[3];		/* indices for final EncodeABC lookup */
 
+
     /* Apply DecodeLMN, MatrixLMN(decode), and MatrixPQR. */
     if (!pjc->skipDecodeLMN)
         cie_lookup_map3(&vec3 /* LMN => PQR */, &pjc->DecodeLMN,
@@ -955,12 +1032,12 @@ gx_cie_real_remap_finish(cie_cached_vector3 vec3, frac * pconc,
  * this is a bug that we will have to address someday.
  */
 static frac
-float2frac_clamp(floatp x)
+float2frac_clamp(double x)
 {
     return float2frac((x <= 0 ? 0 : x >= 1 ? 1 : x));
 }
 int
-gx_cie_xyz_remap_finish(cie_cached_vector3 vec3, frac * pconc,
+gx_cie_xyz_remap_finish(cie_cached_vector3 vec3, frac * pconc, float *xyz,
                         const gs_imager_state * pis,
                         const gs_color_space *pcs)
 {
@@ -974,9 +1051,13 @@ gx_cie_xyz_remap_finish(cie_cached_vector3 vec3, frac * pconc,
         cie_lookup_map3(&vec3 /* LMN => XYZ */, &pjc->DecodeLMN,
                         "Decode/MatrixLMN");
 
-    pconc[0] = float2frac_clamp(cie_cached2float(vec3.u));
-    pconc[1] = float2frac_clamp(cie_cached2float(vec3.v));
-    pconc[2] = float2frac_clamp(cie_cached2float(vec3.w));
+    xyz[0] = cie_cached2float(vec3.u);
+    xyz[1] = cie_cached2float(vec3.v);
+    xyz[2] = cie_cached2float(vec3.w);
+
+    pconc[0] = float2frac_clamp(xyz[0]);
+    pconc[1] = float2frac_clamp(xyz[1]);
+    pconc[2] = float2frac_clamp(xyz[2]);
     return 3;
 }
 
