@@ -69,6 +69,7 @@ int gx_default_get_param(gx_device *dev, char *Param, void *list)
     gs_param_list * plist = (gs_param_list *)list;
     int k, colors = dev->color_info.num_components;
     gs_param_string profile_array[NUM_DEVICE_PROFILES];
+    gs_param_string postren_profile;
     gs_param_string proof_profile, link_profile, icc_colorants;
     gsicc_rendering_intents_t profile_intents[NUM_DEVICE_PROFILES];
     gsicc_blackptcomp_t blackptcomps[NUM_DEVICE_PROFILES];
@@ -81,8 +82,8 @@ int gx_default_get_param(gx_device *dev, char *Param, void *list)
     bool devicegraytok = true;  /* Default if device profile stuct not set */
     bool graydetection = false;
     bool usefastcolor = false;  /* set for unmanaged color */
-    bool sim_overprint = true;  /* By default simulate overprinting */
-    bool prebandthreshold = true;
+    bool sim_overprint = false;  /* By default do not simulate overprinting */
+    bool prebandthreshold = true, temp_bool = false;
 
     if(strcmp(Param, "OutputDevice") == 0){
         gs_param_string dns;
@@ -297,6 +298,12 @@ int gx_default_get_param(gx_device *dev, char *Param, void *list)
                 blackpreserve[k] = dev_profile->rendercond[k].preserve_black;
             }
         }
+        if (dev_profile->postren_profile == NULL) {
+            param_string_from_string(postren_profile, null_str);
+        } else {
+            param_string_from_transient_string(postren_profile,
+                dev_profile->postren_profile->name);
+        }
         if (dev_profile->proof_profile == NULL) {
             param_string_from_string(proof_profile, null_str);
         } else {
@@ -337,6 +344,7 @@ int gx_default_get_param(gx_device *dev, char *Param, void *list)
             blackptcomps[k] = gsBPNOTSPECIFIED;
             blackpreserve[k] = gsBKPRESNOTSPECIFIED;
         }
+        param_string_from_string(postren_profile, null_str);
         param_string_from_string(proof_profile, null_str);
         param_string_from_string(link_profile, null_str);
         param_string_from_string(icc_colorants, null_str);
@@ -355,6 +363,9 @@ int gx_default_get_param(gx_device *dev, char *Param, void *list)
     }
     if (strcmp(Param, "PreBandThreshold") == 0) {
         return param_write_bool(plist, "PreBandThreshold", &prebandthreshold);
+    }  
+    if (strcmp(Param, "PostRenderProfile") == 0) {
+        return param_write_string(plist, "PostRenderProfile", &(postren_profile));
     }
     if (strcmp(Param, "ProofProfile") == 0) {
         return param_write_string(plist,"ProofProfile", &(proof_profile));
@@ -374,7 +385,7 @@ int gx_default_get_param(gx_device *dev, char *Param, void *list)
     if (strcmp(Param, "ImageICCProfile") == 0) {
         return param_write_string(plist,"ImageICCProfile", &(profile_array[2]));
     }
-    if (strcmp(Param, "GraphicICCProfile") == 0) {
+    if (strcmp(Param, "TextICCProfile") == 0) {
         return param_write_string(plist,"TextICCProfile", &(profile_array[3]));
     }
     if (strcmp(Param, "RenderIntent") == 0) {
@@ -413,6 +424,28 @@ int gx_default_get_param(gx_device *dev, char *Param, void *list)
     if (strcmp(Param, "TextKPreserve") == 0) {
         return param_write_int(plist,"TextKPreserve", (const int *) &(blackpreserve[3]));
     }
+    if (strcmp(Param, "FirstPage") == 0) {
+        return param_write_int(plist, "FirstPage", &dev->FirstPage);
+    }
+    if (strcmp(Param, "LastPage") == 0) {
+        return param_write_int(plist, "LastPage", &dev->LastPage);
+    }
+    if (strcmp(Param, "DisablePageHandler") == 0) {
+        temp_bool = dev->DisablePageHandler;
+        return param_write_bool(plist, "DisablePageHandler", &temp_bool);
+    }
+    if (strcmp(Param, "FILTERIMAGE") == 0) {
+        temp_bool = dev->ObjectFilter & FILTERIMAGE;
+        return param_write_bool(plist, "FILTERIMAGE", &temp_bool);
+    }
+    if (strcmp(Param, "FILTERTEXT") == 0) {
+        temp_bool = dev->ObjectFilter & FILTERTEXT;
+        return param_write_bool(plist, "FILTERTEXT", &temp_bool);
+    }
+    if (strcmp(Param, "FILTERVECTOR") == 0) {
+        temp_bool = dev->ObjectFilter & FILTERVECTOR;
+        return param_write_bool(plist, "FILTERVECTOR", &temp_bool);
+    }
 
     return gs_error_undefined;
 }
@@ -427,6 +460,7 @@ gx_default_get_params(gx_device * dev, gs_param_list * plist)
 
     bool seprs = false;
     gs_param_string dns, pcms, profile_array[NUM_DEVICE_PROFILES];
+    gs_param_string postren_profile;
     gs_param_string proof_profile, link_profile, icc_colorants;
     gsicc_rendering_intents_t profile_intents[NUM_DEVICE_PROFILES];
     gsicc_blackptcomp_t blackptcomps[NUM_DEVICE_PROFILES];
@@ -434,8 +468,8 @@ gx_default_get_params(gx_device * dev, gs_param_list * plist)
     bool devicegraytok = true;  /* Default if device profile stuct not set */
     bool graydetection = false;
     bool usefastcolor = false;  /* set for unmanaged color */
-    bool sim_overprint = true;  /* By default simulate overprinting */
-    bool prebandthreshold = true;
+    bool sim_overprint = false;  /* By default do not simulate overprinting */
+    bool prebandthreshold = true, temp_bool;
     int k;
     gs_param_float_array msa, ibba, hwra, ma;
     gs_param_string_array scna;
@@ -508,7 +542,7 @@ gx_default_get_params(gx_device * dev, gs_param_list * plist)
                 blackpreserve[k] = dev_profile->rendercond[k].preserve_black;
             }
         }
-        /* The proof and link profile */
+        /* The proof, link and post render profile */
         if (dev_profile->proof_profile == NULL) {
             param_string_from_string(proof_profile, null_str);
         } else {
@@ -521,6 +555,12 @@ gx_default_get_params(gx_device * dev, gs_param_list * plist)
             param_string_from_transient_string(link_profile,
                                      dev_profile->link_profile->name);
         }
+        if (dev_profile->postren_profile == NULL) {
+            param_string_from_string(postren_profile, null_str);
+        } else {
+            param_string_from_transient_string(postren_profile,
+                dev_profile->postren_profile->name);
+        }
         devicegraytok = dev_profile->devicegraytok;
         graydetection = dev_profile->graydetection;
         usefastcolor = dev_profile->usefastcolor;
@@ -552,6 +592,7 @@ gx_default_get_params(gx_device * dev, gs_param_list * plist)
         param_string_from_string(proof_profile, null_str);
         param_string_from_string(link_profile, null_str);
         param_string_from_string(icc_colorants, null_str);
+        param_string_from_string(postren_profile, null_str);
     }
     /* Transmit the values. */
     /* Standard parameters */
@@ -589,6 +630,7 @@ gx_default_get_params(gx_device * dev, gs_param_list * plist)
         (code = param_write_string(plist,"ImageICCProfile", &(profile_array[2]))) < 0 ||
         (code = param_write_string(plist,"TextICCProfile", &(profile_array[3]))) < 0 ||
         (code = param_write_string(plist,"ProofProfile", &(proof_profile))) < 0 ||
+        (code = param_write_string(plist, "PostRenderProfile", &(postren_profile))) < 0 ||
         (code = param_write_string(plist,"DeviceLinkProfile", &(link_profile))) < 0 ||
         (code = param_write_string(plist,"ICCOutputColors", &(icc_colorants))) < 0 ||
         (code = param_write_int(plist,"RenderIntent", (const int *) (&(profile_intents[0])))) < 0 ||
@@ -636,6 +678,25 @@ gx_default_get_params(gx_device * dev, gs_param_list * plist)
     if (code < 0)
         return code;
 
+    if ((code = param_write_int(plist, "FirstPage", &dev->FirstPage)) < 0)
+        return code;
+    if ((code = param_write_int(plist, "LastPage", &dev->LastPage)) < 0)
+        return code;
+
+    temp_bool = dev->DisablePageHandler;
+    if ((code = param_write_bool(plist, "DisablePageHandler", &temp_bool)) < 0)
+        return code;
+
+    temp_bool = dev->ObjectFilter & FILTERIMAGE;
+    if ((code = param_write_bool(plist, "FILTERIMAGE", &temp_bool)) < 0)
+        return code;
+    temp_bool = dev->ObjectFilter & FILTERTEXT;
+    if ((code = param_write_bool(plist, "FILTERTEXT", &temp_bool)) < 0)
+        return code;
+    temp_bool = dev->ObjectFilter & FILTERVECTOR;
+    if ((code = param_write_bool(plist, "FILTERVECTOR", &temp_bool)) < 0)
+        return code;
+
     /* Fill in color information. */
 
     if (colors > 1) {
@@ -1244,8 +1305,9 @@ gx_default_put_params(gx_device * dev, gs_param_list * plist)
     bool devicegraytok = true;
     bool graydetection = false;
     bool usefastcolor = false;
-    bool sim_overprint = true;
+    bool sim_overprint = false;
     bool prebandthreshold = false;
+    bool temp_bool;
     int  profile_types[NUM_DEVICE_PROFILES] = {gsDEFAULTPROFILE,
                                                gsGRAPHICPROFILE,
                                                gsIMAGEPROFILE,
@@ -1467,6 +1529,9 @@ nce:
     if ((code = param_read_string(plist, "DeviceLinkProfile", &icc_pro)) != 1) {
         gx_default_put_icc(&icc_pro, dev, gsLINKPROFILE);
     }
+    if ((code = param_read_string(plist, "PostRenderProfile", &icc_pro)) != 1) {
+        gx_default_put_icc(&icc_pro, dev, gsPRPROFILE);
+    }
     if ((code = param_read_int(plist, (param_name = "RenderIntent"),
                                                     &(rend_intent[0]))) < 0) {
         ecode = code;
@@ -1699,6 +1764,50 @@ label:\
             ecode = code;
     }
 
+    code = param_read_int(plist, "FirstPage", &dev->FirstPage);
+    if (code < 0)
+        ecode = code;
+
+    code = param_read_int(plist,  "LastPage", &dev->LastPage);
+    if (code < 0)
+        ecode = code;
+
+    code = param_read_bool(plist, "DisablePageHandler", &temp_bool);
+    if (code < 0)
+        ecode = code;
+    if (code == 0)
+        dev->DisablePageHandler = temp_bool;
+
+    code = param_read_bool(plist, "FILTERIMAGE", &temp_bool);
+    if (code < 0)
+        ecode = code;
+    if (code == 0) {
+        if (temp_bool)
+            dev->ObjectFilter |= FILTERIMAGE;
+        else
+            dev->ObjectFilter &= ~FILTERIMAGE;
+    }
+
+    code = param_read_bool(plist, "FILTERTEXT", &temp_bool);
+    if (code < 0)
+        ecode = code;
+    if (code == 0) {
+        if (temp_bool)
+            dev->ObjectFilter |= FILTERTEXT;
+        else
+            dev->ObjectFilter &= ~FILTERTEXT;
+    }
+
+    code = param_read_bool(plist, "FILTERVECTOR", &temp_bool);
+    if (code < 0)
+        ecode = code;
+    if (code == 0) {
+        if (temp_bool)
+            dev->ObjectFilter |= FILTERVECTOR;
+        else
+            dev->ObjectFilter &= ~FILTERVECTOR;
+    }
+
     /* We must 'commit', in order to detect unknown parameters, */
     /* even if there were errors. */
     code = param_commit(plist);
