@@ -211,7 +211,7 @@ void
 cl_cache_load_slot(CL_CACHE *cache, CL_CACHE_SLOT *slot, int64_t pos, byte *data, int len)
 {
     slot->blocknum = pos / cache->block_size;
-    memcpy(slot->base, data, len);
+    memmove(slot->base, data, len);
 }
 
 /* Use our own FILE structure so that, on some platforms, we write and read
@@ -494,8 +494,8 @@ clist_rewind(clist_file_ptr cf, bool discard_data, const char *fname)
              */
 
             /* Opening with "w" mode deletes the contents when closing. */
-            (void)freopen(fname, gp_fmode_wb, f);
-            (void)freopen(fname, fmode, f);
+            f = freopen(fname, gp_fmode_wb, f);
+            ((IFILE *)cf)->f = freopen(fname, fmode, f);
         } else {
             rewind(f);
         }
