@@ -24,6 +24,7 @@
 #include "gscdefs.h"            /* for image type table */
 #include "gxarith.h"
 #include "gxcspace.h"
+#include "gxpcolor.h"
 #include "gxdevice.h"
 #include "gxdevmem.h"           /* must precede gxcldev.h */
 #include "gxcldev.h"
@@ -43,9 +44,7 @@
 #include "gxdevsop.h"
 #include "gscindex.h"
 #include "gsicc_cms.h"
-#include "gxsample.h"
-#include "gximage.h"
-#include "gxfrac.h"
+#include "gximdecode.h"
 
 extern_gx_image_type_table();
 
@@ -71,7 +70,6 @@ palette_has_color(const gs_color_space *pcs, const gs_pixel_image_t * const pim)
     int num_entries = 1 << bps;
     int k;
     byte psrc[4];
-    int code;
 
     switch(base_type) {
 
@@ -122,7 +120,7 @@ palette_has_color(const gs_color_space *pcs, const gs_pixel_image_t * const pim)
     }
     /* Now go through the palette with the check color function */
     for (k = 0; k < num_entries; k++) {
-        code = gs_cspace_indexed_lookup_bytes(pcs, (float) k, psrc);
+        (void)gs_cspace_indexed_lookup_bytes(pcs, (float) k, psrc); /* this always returns 0 */
         if (!is_neutral(psrc, 1)) {
             /* Has color end this now */
             return true;
@@ -149,9 +147,7 @@ clist_fill_mask(gx_device * dev,
     int orig_x = rx;            /* ditto */
     int orig_width = rwidth;    /* ditto */
     int orig_height = rheight;  /* ditto */
-    int log2_depth = ilog2(depth);
     int y0;
-    int data_x_bit;
     byte copy_op =
         (depth > 1 ? cmd_op_copy_color_alpha :
          cmd_op_copy_mono_planes + cmd_copy_ht_color);  /* Plane not needed here */
@@ -188,7 +184,6 @@ clist_fill_mask(gx_device * dev,
 
     if (cmd_check_clip_path(cdev, pcpath))
         cmd_clear_known(cdev, clip_path_known);
-    data_x_bit = data_x << log2_depth;
     if (cdev->permanent_error < 0)
       return (cdev->permanent_error);
     /* If needed, update the trans_bbox */
@@ -353,12 +348,8 @@ typedef struct clist_image_enum_s {
     int y;
     bool color_map_is_known;
     bool monitor_color;
-    int bps;
-    int spp;
-    SAMPLE_UNPACK_PROC((*unpack));
+    image_decode_t decode;
     byte *buffer;  /* needed for unpacking during monitoring */
-    int spread;
-    sample_map map[GS_IMAGE_MAX_COMPONENTS];
 } clist_image_enum;
 gs_private_st_suffix_add4(st_clist_image_enum, clist_image_enum,
                           "clist_image_enum", clist_image_enum_enum_ptrs,
@@ -378,7 +369,7 @@ row_has_color(byte *data_ptr, clist_image_enum *pie_c, int data_size, int width)
 {
     clist_color_space_t pclcs = pie_c->color_space;
     bool ((*is_neutral)(void*, int));
-    int step_size = data_size * pie_c->spp;
+    int step_size = data_size * pie_c->decode.spp;
     byte *ptr;
     bool is_mono;
     int k;
@@ -409,152 +400,6 @@ row_has_color(byte *data_ptr, clist_image_enum *pie_c, int data_size, int width)
     return false;
 }
 
-/* We need to have the unpacking proc so that we can monitor the data. */
-static void
-get_unpack_proc(clist_image_enum *pie, const float *decode) {
-
-static sample_unpack_proc_t procs[2][6] = {
-    {   sample_unpack_1, sample_unpack_2,
-        sample_unpack_4, sample_unpack_8,
-        0, 0
-    },
-    {   sample_unpack_1_interleaved, sample_unpack_2_interleaved,
-        sample_unpack_4_interleaved, sample_unpack_8_interleaved,
-        0, 0
-    }};
-    int num_planes = pie->num_planes;
-    bool interleaved = (num_planes == 1 && pie->plane_depths[0] != pie->bps);
-    int i;
-    int index_bps = (pie->bps < 8 ? pie->bps >> 1 : (pie->bps >> 2) + 1);
-    gs_image_format_t format = pie->format;
-    int log2_xbytes = (pie->bps <= 8 ? 0 : arch_log2_sizeof_frac);
-
-    switch (format) {
-        case gs_image_format_chunky:
-            pie->spread = 1 << log2_xbytes;
-            break;
-        case gs_image_format_component_planar:
-            pie->spread = (pie->spp) << log2_xbytes;
-            break;
-        case gs_image_format_bit_planar:
-            pie->spread = (pie->spp) << log2_xbytes;
-            break;
-        default:
-           pie->spread = 0;
-    }
-
-    procs[0][4] = procs[1][4] = sample_unpack_12_proc;
-    procs[0][5] = procs[1][5] = sample_unpackicc_16_proc;
-    if (interleaved) {
-        int num_components = pie->plane_depths[0] / pie->bps;
-
-        for (i = 1; i < num_components; i++) {
-            if (decode[0] != decode[i * 2 + 0] ||
-                decode[1] != decode[i * 2 + 1])
-                break;
-        }
-        if (i == num_components)
-            interleaved = false; /* Use single table. */
-    }
-    pie->unpack = procs[interleaved][index_bps];
-}
-
-/* We also need the mapping method for the unpacking proc */
-static void
-get_map(clist_image_enum *pie, gs_image_format_t format, const float *decode)
-{
-    int ci, decode_type;
-    int bps = pie->bps;
-    int spp = pie->spp;
-    static const float default_decode[] = {
-        0.0, 1.0, 0.0, 1.0, 0.0, 1.0, 0.0, 1.0, 0.0, 1.0
-    };
-
-    decode_type = 3; /* 0=custom, 1=identity, 2=inverted, 3=impossible */
-    for (ci = 0; ci < spp; ci +=2 ) {
-        decode_type &= (decode[ci] == 0. && decode[ci + 1] == 1.) |
-                       (decode[ci] == 1. && decode[ci + 1] == 0.) << 1;
-    }
-
-    /* Initialize the maps from samples to intensities. */
-    for (ci = 0; ci < spp; ci++) {
-        sample_map *pmap = &pie->map[ci];
-
-        /* If the decoding is [0 1] or [1 0], we can fold it */
-        /* into the expansion of the sample values; */
-        /* otherwise, we have to use the floating point method. */
-
-        const float *this_decode = &decode[ci * 2];
-        const float *map_decode;        /* decoding used to */
-                                        /* construct the expansion map */
-        const float *real_decode;       /* decoding for expanded samples */
-
-        map_decode = real_decode = this_decode;
-        if (!(decode_type & 1)) {
-            if ((decode_type & 2) && bps <= 8) {
-                real_decode = default_decode;
-            } else {
-                map_decode = default_decode;
-            }
-        }
-        if (bps > 2 || format != gs_image_format_chunky) {
-            if (bps <= 8)
-                image_init_map(&pmap->table.lookup8[0], 1 << bps,
-                               map_decode);
-        } else {                /* The map index encompasses more than one pixel. */
-            byte map[4];
-            register int i;
-
-            image_init_map(&map[0], 1 << bps, map_decode);
-            switch (bps) {
-                case 1:
-                    {
-                        register bits32 *p = &pmap->table.lookup4x1to32[0];
-
-                        if (map[0] == 0 && map[1] == 0xff)
-                            memcpy((byte *) p, lookup4x1to32_identity, 16 * 4);
-                        else if (map[0] == 0xff && map[1] == 0)
-                            memcpy((byte *) p, lookup4x1to32_inverted, 16 * 4);
-                        else
-                            for (i = 0; i < 16; i++, p++)
-                                ((byte *) p)[0] = map[i >> 3],
-                                    ((byte *) p)[1] = map[(i >> 2) & 1],
-                                    ((byte *) p)[2] = map[(i >> 1) & 1],
-                                    ((byte *) p)[3] = map[i & 1];
-                    }
-                    break;
-                case 2:
-                    {
-                        register bits16 *p = &pmap->table.lookup2x2to16[0];
-
-                        for (i = 0; i < 16; i++, p++)
-                            ((byte *) p)[0] = map[i >> 2],
-                                ((byte *) p)[1] = map[i & 3];
-                    }
-                    break;
-            }
-        }
-        pmap->decode_base /* = decode_lookup[0] */  = real_decode[0];
-        pmap->decode_factor =
-            (real_decode[1] - real_decode[0]) /
-            (bps <= 8 ? 255.0 : (float)frac_1);
-        pmap->decode_max /* = decode_lookup[15] */  = real_decode[1];
-        if (decode_type) {
-            pmap->decoding = sd_none;
-            pmap->inverted = map_decode[0] != 0;
-        } else if (bps <= 4) {
-            int step = 15 / ((1 << bps) - 1);
-            int i;
-
-            pmap->decoding = sd_lookup;
-            for (i = 15 - step; i > 0; i -= step)
-                pmap->decode_lookup[i] = pmap->decode_base +
-                    i * (255.0 / 15) * pmap->decode_factor;
-        } else
-            pmap->decoding = sd_compute;
-    }
-}
-
 /* Forward declarations */
 static bool image_band_box(gx_device * dev, const clist_image_enum * pie,
                             int y, int h, gs_int_rect * pbox);
@@ -653,7 +498,7 @@ clist_begin_typed_image(gx_device * dev, const gs_imager_state * pis,
     bool bp_changed = false;
     cmm_dev_profile_t *dev_profile = NULL;
     cmm_profile_t *gs_output_profile;
-    bool is_planar_dev = dev_proc(dev, dev_spec_op)(dev, gxdso_is_native_planar, NULL, 0) > 0;
+    bool is_planar_dev = dev->is_planar;
     bool render_is_valid;
     int csi;
 
@@ -752,8 +597,8 @@ clist_begin_typed_image(gx_device * dev, const gs_imager_state * pis,
         int bytes_per_plane, bytes_per_row;
 
         bits_per_pixel = pim->BitsPerComponent * num_components;
-        pie->bps = bits_per_pixel/num_components;
-        pie->spp = num_components;
+        pie->decode.bps = bits_per_pixel/num_components;
+        pie->decode.spp = num_components;
         pie->image = *pim;
         pie->dcolor = *pdcolor;
         if (prect)
@@ -898,6 +743,7 @@ clist_begin_typed_image(gx_device * dev, const gs_imager_state * pis,
                 pie->color_space.icc_info.icc_num_components =
                     src_profile->num_comps;
                 pie->color_space.icc_info.is_lab = src_profile->islab;
+                pie->color_space.icc_info.default_match = src_profile->default_match;
                 pie->color_space.icc_info.data_cs = src_profile->data_cs;
                 src_profile->rend_cond = stored_rendering_cond;
                 render_is_valid = src_profile->rend_is_valid;
@@ -918,7 +764,7 @@ clist_begin_typed_image(gx_device * dev, const gs_imager_state * pis,
             goto use_default;
     }
     if (pim->Interpolate) {
-        if (strcmp("pattern-clist",dev->dname) == 0)
+        if (gx_device_is_pattern_clist(dev))
             goto use_default;
         pie->support.x = pie->support.y = MAX_ISCALE_SUPPORT + 1;
     } else {
@@ -953,25 +799,27 @@ clist_begin_typed_image(gx_device * dev, const gs_imager_state * pis,
     }
     /* Decide if we need to do any monitoring of the colors.  Note that multiple source
        (planes) is treated as color */
-    pie->unpack = NULL;
+    pie->decode.unpack = NULL;
     if (dev_profile->pageneutralcolor && pie->color_space.icc_info.data_cs != gsGRAY) {
         /* If it is an index image, then check the pallete only */
         if (!indexed) {
             pie->monitor_color = true;
             /* Set up the unpacking proc for monitoring */
-            get_unpack_proc(pie, pim->Decode);
-            get_map(pie, pim->format, pim->Decode);
-            if (pie->unpack == NULL) {
+            get_unpack_proc((gx_image_enum_common_t*) pie, &(pie->decode), 
+                             pim->format, pim->Decode);
+            get_map(&(pie->decode), pim->format, pim->Decode);
+            if (pie->decode.unpack == NULL) {
                 /* If we cant unpack, then end monitoring now. Treat as has color */
                 dev_profile->pageneutralcolor = false;
                 gsicc_mcm_end_monitor(pis->icc_link_cache, dev);
             } else {
                 /* We need to allocate the buffer for unpacking during monitoring.
                     This is mainly for the 12bit case */
-                int bsize = ((pie->bps > 8 ? (pim->Width) * 2 : pim->Width) + 15) * num_components;
+                int bsize = ((pie->decode.bps > 8 ? (pim->Width) * 2 : pim->Width) + 15) * num_components;
                 pie->buffer = gs_alloc_bytes(mem, bsize, "image buffer");
                 if (pie->buffer == 0) {
-                    gs_free_object(mem, pie->buffer, "clist_begin_typed_image");
+                    gs_free_object(mem, pie, "clist_begin_typed_image");
+                    *pinfo = NULL;
                     return_error(gs_error_VMerror);
                 }
             }
@@ -1009,43 +857,30 @@ clist_begin_typed_image(gx_device * dev, const gs_imager_state * pis,
     if (!masked) {
         /*
          * Calculate (conservatively) the set of colors that this image
-         * might generate.  For single-component images with up to 4 bits
-         * per pixel, standard Decode values, and no Interpolate, we
-         * generate all the possible colors now; otherwise, we assume that
-         * any color might be generated.  It is possible to do better than
-         * this, but we won't bother unless there's evidence that it's
-         * worthwhile.
+         * might generate.  For single-component images we can sample
+         * this. We generate all the possible colors now; otherwise,
+         * we assume that any color might be generated.  It is possible
+         * to do better than this, but we won't bother unless there's
+         * evidence that it's worthwhile.
          */
         gx_color_usage_bits all = gx_color_usage_all(cdev);
 
-        if (bits_per_pixel > 4 || pim->Interpolate || num_components > 1)
+        if (num_components > 1)
             color_usage = all;
         else {
-            int max_value = (1 << bits_per_pixel) - 1;
-            float dmin = pim->Decode[0], dmax = pim->Decode[1];
-            float dtemp;
+            const gs_color_space *pcs = pim->ColorSpace;
+            cs_proc_remap_color((*remap_color)) = pcs->type->remap_color;
+            gs_client_color cc;
+            gx_drawing_color dcolor;
+            int i;
+            int max_value = indexed ? pcs->params.indexed.hival : 1;
 
-            if (dmax < dmin)
-                dtemp = dmax, dmax = dmin, dmin = dtemp;
-            if (dmin != 0 ||
-                dmax != (indexed ? max_value : 1)
-                ) {
-                color_usage = all;
-            } else {
-                /* Enumerate the possible pixel values. */
-                const gs_color_space *pcs = pim->ColorSpace;
-                cs_proc_remap_color((*remap_color)) = pcs->type->remap_color;
-                gs_client_color cc;
-                gx_drawing_color dcolor;
-                int i;
-                double denom = (indexed ? 1 : max_value);
-
-                for (i = 0; i <= max_value; ++i) {
-                    cc.paint.values[0] = (double)i / denom;
-                    remap_color(&cc, pcs, &dcolor, pis, dev,
-                                gs_color_select_source);
-                    color_usage |= cmd_drawing_color_usage(cdev, &dcolor);
-                }
+            for (i = 0; i <= max_value; ++i) {
+                /* Enumerate the indexed colors, or just Black (DeviceGray = 0) */
+                cc.paint.values[0] = (double)i;
+                remap_color(&cc, pcs, &dcolor, pis, dev,
+                            gs_color_select_source);
+                color_usage |= cmd_drawing_color_usage(cdev, &dcolor);
             }
         }
     }
@@ -1064,8 +899,8 @@ clist_begin_typed_image(gx_device * dev, const gs_imager_state * pis,
         if (pcpath) {
             gs_fixed_rect obox;
             gx_cpath_outer_box(pcpath, &obox);
-            pie->ymin = max(y0, fixed2int(obox.p.y));
-            pie->ymax = min(y1, fixed2int(obox.q.y));
+            pie->ymin = max(0, max(y0, fixed2int(obox.p.y)));
+            pie->ymax = min(min(y1, fixed2int(obox.q.y)), dev->height);
         } else {
             pie->ymin = max(y0, 0);
             pie->ymax = min(y1, dev->height);
@@ -1098,6 +933,7 @@ use_default:
     if (pie != NULL)
         gs_free_object(mem, pie->buffer, "clist_begin_typed_image");
     gs_free_object(mem, pie, "clist_begin_typed_image");
+    *pinfo = NULL;
 
     if (pis->has_transparency){
         return -1;
@@ -1277,6 +1113,13 @@ clist_image_plane_data(gx_image_enum_common_t * info,
                 goto error_in_rect;
             if (pie->uses_color) {
                 do {
+                    /* We want to write the color taking into account the entire image so */
+                    /* we set re.rect_nbands from pie->ymin and pie->ymax so that we will */
+                    /* make the decision to write 'all_bands' the same for the whole image */
+                    /* This is slightly more efficient, and is required for patterns with */
+                    /* transparency that push the group at the begin_image step.          */
+                    re.rect_nbands = ((pie->ymax + re.band_height - 1) / re.band_height) -
+                                     ((pie->ymin) / re.band_height);
                     code = cmd_put_drawing_color(cdev, re.pcls, &pie->dcolor,
                                                  &re, devn_not_tile);
                 } while (RECT_RECOVER(code));
@@ -1765,7 +1608,6 @@ cmd_put_color_mapping(gx_device_clist_writer * cldev,
     /* Now put out the transfer functions. */
     {
         uint which = 0;
-        bool all_same = true;
         bool send_default_comp = false;
         int i;
         gs_id default_comp_id, xfer_ids[4];
@@ -1789,8 +1631,6 @@ cmd_put_color_mapping(gx_device_clist_writer * cldev,
         for (i = 0; i < countof(cldev->transfer_ids); ++i) {
             if (xfer_ids[i] != cldev->transfer_ids[i])
                 which |= 1 << i;
-            if (xfer_ids[i] != default_comp_id)
-                all_same = false;
             if (xfer_ids[i] == default_comp_id &&
                 cldev->transfer_ids[i] != default_comp_id)
                 send_default_comp = true;
@@ -1851,7 +1691,7 @@ cmd_put_color_mapping(gx_device_clist_writer * cldev,
 #define I_FLOOR(x) ((int)floor(x))
 #define I_CEIL(x) ((int)ceil(x))
 static void
-box_merge_point(gs_int_rect * pbox, floatp x, floatp y)
+box_merge_point(gs_int_rect * pbox, double x, double y)
 {
     int t;
 
@@ -1984,12 +1824,12 @@ image_band_box(gx_device * dev, const clist_image_enum * pie, int y, int h,
                 if_debug3m('b', dev->memory, "   (px) t=%g => (%d,%g)\n",
                            t, px, pa.y + t * dy);
                 if (in_range(t, pa.y + t * dy, py, qy))
-                    box_merge_point(pbox, (floatp) px, t);
+                    box_merge_point(pbox, (double) px, t);
                 t = (qx - pa.x) / dx;
                 if_debug3m('b', dev->memory, "   (qx) t=%g => (%d,%g)\n",
                            t, qx, pa.y + t * dy);
                 if (in_range(t, pa.y + t * dy, py, qy))
-                    box_merge_point(pbox, (floatp) qx, t);
+                    box_merge_point(pbox, (double) qx, t);
             }
             if (dy != 0) {
                 double t = (py - pa.y) / dy;
@@ -1997,12 +1837,12 @@ image_band_box(gx_device * dev, const clist_image_enum * pie, int y, int h,
                 if_debug3m('b', dev->memory, "   (py) t=%g => (%g,%d)\n",
                            t, pa.x + t * dx, py);
                 if (in_range(t, pa.x + t * dx, px, qx))
-                    box_merge_point(pbox, t, (floatp) py);
+                    box_merge_point(pbox, t, (double) py);
                 t = (qy - pa.y) / dy;
                 if_debug3m('b', dev->memory, "   (qy) t=%g => (%g,%d)\n",
                            t, pa.x + t * dx, qy);
                 if (in_range(t, pa.x + t * dx, px, qx))
-                    box_merge_point(pbox, t, (floatp) qy);
+                    box_merge_point(pbox, t, (double) qy);
             }
 #undef in_range
         }
@@ -2171,9 +2011,9 @@ cmd_image_plane_data_mon(gx_device_clist_writer * cldev, gx_clist_state * pcls,
     int plane, i;
     int code;
     int width = pie_c->rect.q.x - pie_c->rect.p.x;
-    int dsize = (((width + (planes[0]).data_x) * pie_c->spp *
-                   pie_c->bps / pie->num_planes + 7) >> 3);
-    int data_size = pie_c->spread / pie->num_planes;
+    int dsize = (((width + (planes[0]).data_x) * pie_c->decode.spp *
+        pie_c->decode.bps / pie->num_planes + 7) >> 3);
+    int data_size = pie_c->decode.spread / pie->num_planes;
 
     *found_color = false;
 
@@ -2195,16 +2035,17 @@ cmd_image_plane_data_mon(gx_device_clist_writer * cldev, gx_clist_state * pcls,
             /* Here we need to unpack and actually look at the image data
                to see if we have any non-neutral colors */
             int pdata_x;
-            byte *data_ptr = planes[0].data + i * planes[0].raster + offsets[0] + offset;
-            byte *buffer = (*pie_c->unpack)(pie_c->buffer, &pdata_x, data_ptr, 0, dsize, pie_c->map,
-                                            pie_c->spread, pie_c->spp);
+            byte *data_ptr =  (byte *)(planes[0].data + i * planes[0].raster + offsets[0] + offset);
+            byte *buffer = (byte *)(*pie_c->decode.unpack)(pie_c->buffer, &pdata_x, 
+                                     data_ptr, 0, dsize, pie_c->decode.map,
+                pie_c->decode.spread, pie_c->decode.spp);
 
             for (plane = 1; plane < pie->num_planes; ++plane) {
                 /* unpack planes after the first (if any), relying on spread to place the */
                 /* data at the correct spacing, with the buffer start adjusted for each plane */
-                data_ptr = planes[plane].data + i * planes[plane].raster + offsets[plane] + offset;
-                (*pie_c->unpack)(pie_c->buffer + (data_size * plane), &pdata_x, data_ptr, 0,
-                                 dsize, pie_c->map, pie_c->spread, pie_c->spp);
+                data_ptr = (byte *)(planes[plane].data + i * planes[plane].raster + offsets[plane] + offset);
+                (*pie_c->decode.unpack)(pie_c->buffer + (data_size * plane), &pdata_x, data_ptr, 0,
+                    dsize, pie_c->decode.map, pie_c->decode.spread, pie_c->decode.spp);
             }
             if (row_has_color(buffer, pie_c, data_size, width)) {
                 /* Has color.  We are done monitoring */
