@@ -1014,6 +1014,11 @@ gs_fapi_finish_render(gs_font *pfont, gs_state *pgs, gs_text_enum_t *penum, gs_f
      * non-cacheing, non-marking cases, we must not draw the glyph.
      */
     if (pgs->in_charpath && !SHOW_IS(penum, TEXT_DO_NONE)) {
+        gs_point pt;
+
+        if ((code = gs_currentpoint(penum_pgs, &pt)) < 0)
+            return code;
+
         if ((code =
              outline_char(mem, I, import_shift_v, penum_s, penum_pgs->path,
                           !pbfont->PaintType)) < 0) {
@@ -1040,8 +1045,15 @@ gs_fapi_finish_render(gs_font *pfont, gs_state *pgs, gs_text_enum_t *penum, gs_f
             gs_imager_state *pis = (gs_imager_state *) penum_pgs->show_gstate;
             gs_point pt;
 
+            /* This mimics code which is used below in the case where I->Use_outline is set.
+             * This is usually caused by setting -dTextAlphaBits, and will result in 'undefinedresult'
+             * errors. We want the code to work the same regardless off the setting of -dTextAlphaBits
+             * and it seems the simplest way to provoke the same error is to do the same steps....
+             * Note that we do not actually ever use the returned value.
+             */
             if ((code = gs_currentpoint(penum_pgs, &pt)) < 0)
                 return code;
+
             if ((code =
                  outline_char(mem, I, import_shift_v, penum_s,
                               penum_pgs->path, !pbfont->PaintType)) < 0)
@@ -1067,10 +1079,12 @@ gs_fapi_finish_render(gs_font *pfont, gs_state *pgs, gs_text_enum_t *penum, gs_f
 
                 penum_pgs->fill_adjust.x = penum_pgs->fill_adjust.y = 0;
 
-                if ((code = gs_fill(penum_pgs)) < 0)
-                    return code;
+                code = gs_fill(penum_pgs);
 
                 penum_pgs->in_cachedevice = in_cachedevice;
+
+                if (code < 0)
+                    return code;
             }
             if ((code = gs_moveto(penum_pgs, pt.x, pt.y)) < 0)
                 return code;
@@ -1078,6 +1092,16 @@ gs_fapi_finish_render(gs_font *pfont, gs_state *pgs, gs_text_enum_t *penum, gs_f
         else {
             int rast_orig_x = rast.orig_x;
             int rast_orig_y = -rast.orig_y;
+            gs_point pt;
+
+            /* This mimics code which is used above in the case where I->Use_outline is set.
+             * This is usually caused by setting -dTextAlphaBits, and will result in 'undefinedresult'
+             * errors. We want the code to work the same regardless off the setting of -dTextAlphaBits
+             * and it seems the simplest way to provoke the same error is to do the same steps....
+             * Note that we do not actually ever use the returned value.
+             */
+            if ((code = gs_currentpoint(penum_pgs, &pt)) < 0)
+                return code;
 
             if (penum_pgs->in_cachedevice == CACHE_DEVICE_CACHING) {    /* Using GS cache */
                 /*  GS and renderer may transform coordinates few differently.
@@ -1195,7 +1219,6 @@ gs_fapi_do_char(gs_font *pfont, gs_state *pgs, gs_text_enum_t *penum, char *font
     int code;
     bool imagenow = false;
     bool align_to_pixels = gs_currentaligntopixels(pbfont->dir);
-    gs_fapi_descendant_code fdc;
     gs_memory_t *mem = pfont->memory;
     enum
     {
@@ -1343,25 +1366,17 @@ gs_fapi_do_char(gs_font *pfont, gs_state *pgs, gs_text_enum_t *penum, char *font
                 font_scale.matrix[3] = 1;
         }
 
-        if (!bCID
-            || (pbfont->FontType != ft_encrypted
-                && pbfont->FontType != ft_encrypted2)) {
-            fdc = gs_fapi_toplevel_prepared;
-        }
-        else {
-            fdc = gs_fapi_toplevel_prepared;
-        }
         if ((code =
              gs_fapi_renderer_retcode(mem, I,
                                       I->get_scaled_font(I, &I->ff,
                                                          &font_scale, NULL,
-                                                         fdc))) < 0) {
+                                                         gs_fapi_toplevel_prepared))) < 0) {
             return code;
         }
     }
 
     cr.char_codes_count = 1;
-    cr.char_codes[0] = (index == GS_NO_GLYPH) ? 0 : index;
+    cr.char_codes[0] = index;
     cr.client_char_code = chr;
     cr.is_glyph_index = true;
     enc_char_name_string.data = NULL;
@@ -1414,7 +1429,7 @@ gs_fapi_do_char(gs_font *pfont, gs_state *pgs, gs_text_enum_t *penum, char *font
                 cr.sb_x = fapi_round(sbw[2] / 2 * scale);
                 cr.sb_y = fapi_round(pbfont->FontBBox.q.y * scale);
                 cr.aw_y = fapi_round(-pbfont->FontBBox.q.x * scale);    /* Sic ! */
-                cr.metrics_scale = (bIsType1GlyphData ? 1000 : 1);
+                cr.metrics_scale = 1;
                 cr.metrics_type = gs_fapi_metrics_replace;
                 sbw[0] = sbw[2] / 2;
                 sbw[1] = pbfont->FontBBox.q.y;
