@@ -333,16 +333,9 @@ gx_overprint_generic_fill_rectangle(
                     }
                 }
             } else {
-                if (blendspot) {
-                    for (j = 0, comps = drawn_comps; comps != 0; ++j, comps >>= 1) {
-                        if ((comps & 0x1) != 0)
-                            dest_cvals[j] = src_cvals[j];
-                    }
-                } else {
-                    for (j = 0, comps = drawn_comps; comps != 0; ++j, comps >>= 1) {
-                        if ((comps & 0x1) != 0)
-                            dest_cvals[j] = src_cvals[j];
-                    }
+                for (j = 0, comps = drawn_comps; comps != 0; ++j, comps >>= 1) {
+                    if ((comps & 0x1) != 0)
+                        dest_cvals[j] = src_cvals[j];
                 }
             }
             *cp = dev_proc(tdev, encode_color)(tdev, dest_cvals);
@@ -612,23 +605,10 @@ gx_overprint_sep_fill_rectangle_2(
                                                    0 );
         if (code < 0)
             break;
-        if (blendspot) {
-            /* We need to blend the CMYK colorants as we are simulating
-               the overprint of a spot colorant with its equivalent CMYK
-               colorants */
-            for (i = 0, j = 0; i < byte_w; i++, cp++) {
-                int temp = (255-*cp) * (255-pcolor[j]);
-                temp = temp >> 8;
-                *cp = (255-temp);
-                if (++j == byte_depth)
-                    j = 0;
-            }
-        } else {
-            for (i = 0, j = 0; i < byte_w; i++, cp++) {
-                *cp = (*cp & pmask[j]) | pcolor[j];
-                if (++j == byte_depth)
-                    j = 0;
-            }
+        for (i = 0, j = 0; i < byte_w; i++, cp++) {
+            *cp = (*cp & pmask[j]) | pcolor[j];
+            if (++j == byte_depth)
+                j = 0;
         }
         code = dev_proc(tdev, copy_color)( tdev,
                                            gb_buff,
