@@ -98,8 +98,8 @@ static dev_proc_fill_rectangle_hl_color(pattern_accum_fill_rectangle_hl_color);
 
 /* The device descriptor */
 static const gx_device_pattern_accum gs_pattern_accum_device =
-{std_device_std_body_open(gx_device_pattern_accum, 0,
-                          "pattern accumulator",
+{std_device_std_body_type_open(gx_device_pattern_accum, 0,
+                          "pattern accumulator", &st_device_pattern_accum,
                           0, 0, 72, 72),
  {
      /* NOTE: all drawing procedures must be defaulted, not forwarded. */
@@ -779,6 +779,8 @@ gx_pattern_cache_free_entry(gx_pattern_cache * pcache, gx_color_tile * ctile)
                             "gx_pattern_cache_free_entry");
             ctile->cdev->common.icc_cache_cl = NULL;
             ctile->cdev->writer.pinst = NULL;
+            gs_free_object(ctile->cdev->common.memory->non_gc_memory, ctile->cdev->common.cache_chunk, "free tile cache for clist");
+            ctile->cdev->common.cache_chunk = 0;
             temp_device = (gx_device *)ctile->cdev;
             gx_device_retain(temp_device, false);
             ctile->cdev = NULL;
