@@ -127,8 +127,6 @@ static const gs_param_item_t pdf_param_items[] = {
     pi("PDFUseOldCMS", gs_param_type_bool, UseOldColor),
     pi("FastWebView", gs_param_type_bool, Linearise),
     pi("NoOutputFonts", gs_param_type_bool, FlattenFonts),
-    pi("FirstPage", gs_param_type_int, FirstPage),
-    pi("LastPage", gs_param_type_int, LastPage),
 #undef pi
     gs_param_item_end
 };
@@ -396,6 +394,7 @@ gdev_pdf_put_params_impl(gx_device * dev, const gx_device_pdf * save_dev, gs_par
         default:
             ecode = code;
             param_signal_error(plist, param_name, ecode);
+            break;
         case 0:
             if (!(locked && pdev->params.LockDistillerParams)) {
                 /*
@@ -619,7 +618,7 @@ gdev_pdf_put_params_impl(gx_device * dev, const gx_device_pdf * save_dev, gs_par
     } else {
         if ((pdev->params.ColorConversionStrategy == ccs_CMYK &&
              strcmp(pdev->color_info.cm_name, "DeviceCMYK")) ||
-            ((pdev->params.ColorConversionStrategy == ccs_sRGB ||
+            ((pdev->params.ColorConversionStrategy == ccs_RGB ||
              pdev->params.ColorConversionStrategy == ccs_sRGB) &&
               strcmp(pdev->color_info.cm_name, "DeviceRGB")) ||
             (pdev->params.ColorConversionStrategy == ccs_Gray &&
@@ -932,10 +931,6 @@ pdf_dsc_process(gx_device_pdf * pdev, const gs_param_string_array * pma)
             }
             continue;
         }
-
-        if (pdev->ParseDSCCommentsForDocInfo || pdev->PreserveEPSInfo)
-            code = cos_dict_put_c_key_string(pdev->Info, key,
-                                             pvalue->data, pvalue->size);
     }
     return code;
 }
