@@ -168,7 +168,7 @@ copy_ps_file_stripping_all(stream *s, const char *fname, bool HaveTrueTypes)
     int n, l = 0, m = sizeof(buf) - 1, outl = 0;
     bool skipping = false;
 
-    f = sfopen(fname, "rb", s->memory);
+    f = sfopen(fname, "r", s->memory);
     if (f == NULL)
         return_error(gs_error_undefinedfilename);
     n = sfread(buf, 1, m, f);
@@ -244,7 +244,7 @@ copy_ps_file_strip_comments(stream *s, const char *fname, bool HaveTrueTypes)
     int n, l = 0, m = sizeof(buf) - 1, outl = 0;
     bool skipping = false;
 
-    f = sfopen(fname, "rb", s->memory);
+    f = sfopen(fname, "r", s->memory);
     if (f == NULL)
         return_error(gs_error_undefinedfilename);
     n = sfread(buf, 1, m, f);
@@ -648,10 +648,12 @@ pdf_obj_mark_unused(gx_device_pdf *pdev, long id)
     int64_t tpos = gp_ftell_64(tfile);
     gs_offset_t pos = 0;
 
-    gp_fseek_64 (tfile, ((int64_t)(id - pdev->FirstObjectNumber)) * sizeof(pos),
-          SEEK_SET);
+    if (gp_fseek_64 (tfile, ((int64_t)(id - pdev->FirstObjectNumber)) * sizeof(pos),
+          SEEK_SET) != 0)
+          return gs_error_ioerror;
     fwrite(&pos, sizeof(pos), 1, tfile);
-    gp_fseek_64(tfile, tpos, SEEK_SET);
+    if (gp_fseek_64(tfile, tpos, SEEK_SET) != 0)
+        return gs_error_ioerror;
     return 0;
 }
 
@@ -668,10 +670,12 @@ pdf_open_obj(gx_device_pdf * pdev, long id, pdf_resource_type_t type)
         FILE *tfile = pdev->xref.file;
         int64_t tpos = gp_ftell_64(tfile);
 
-        gp_fseek_64 (tfile, ((int64_t)(id - pdev->FirstObjectNumber)) * sizeof(pos),
-              SEEK_SET);
+        if (gp_fseek_64 (tfile, ((int64_t)(id - pdev->FirstObjectNumber)) * sizeof(pos),
+              SEEK_SET) != 0)
+              return gs_error_ioerror;
         fwrite(&pos, sizeof(pos), 1, tfile);
-        gp_fseek_64(tfile, tpos, SEEK_SET);
+        if (gp_fseek_64(tfile, tpos, SEEK_SET) != 0)
+              return gs_error_ioerror;
     }
     if (pdev->ForOPDFRead && pdev->ProduceDSC) {
         switch(type) {
@@ -1157,8 +1161,8 @@ pdf_cancel_resource(gx_device_pdf * pdev, pdf_resource_t *pres, pdf_resource_typ
     if (pres->object) {
         pres->object->written = true;
         if (rtype == resourceXObject || rtype == resourceCharProc || rtype == resourceOther
-            || rtype > NUM_RESOURCE_TYPES) {
-            int code = cos_stream_release_pieces((cos_stream_t *)pres->object);
+            || rtype >= NUM_RESOURCE_TYPES) {
+            int code = cos_stream_release_pieces(pdev, (cos_stream_t *)pres->object);
 
             if (code < 0)
                 return code;
@@ -1469,7 +1473,7 @@ pdf_begin_resource_body(gx_device_pdf * pdev, pdf_resource_type_t rtype,
 {
     int code;
 
-    if (rtype > NUM_RESOURCE_TYPES)
+    if (rtype >= NUM_RESOURCE_TYPES)
         rtype = resourceOther;
 
     code = pdf_begin_aside(pdev, PDF_RESOURCE_CHAIN(pdev, rtype, rid),
@@ -1485,7 +1489,7 @@ pdf_begin_resource(gx_device_pdf * pdev, pdf_resource_type_t rtype, gs_id rid,
 {
     int code;
 
-    if (rtype > NUM_RESOURCE_TYPES)
+    if (rtype >= NUM_RESOURCE_TYPES)
         rtype = resourceOther;
 
     code = pdf_begin_resource_body(pdev, rtype, rid, ppres);
@@ -1517,7 +1521,7 @@ pdf_alloc_resource(gx_device_pdf * pdev, pdf_resource_type_t rtype, gs_id rid,
 {
     int code;
 
-    if (rtype > NUM_RESOURCE_TYPES)
+    if (rtype >= NUM_RESOURCE_TYPES)
         rtype = resourceOther;
 
     code = pdf_alloc_aside(pdev, PDF_RESOURCE_CHAIN(pdev, rtype, rid),
@@ -1649,7 +1653,7 @@ pdf_store_page_resources(gx_device_pdf *pdev, pdf_page_t *page, bool clear_usage
         stream *s = 0;
         int j;
 
-        if (i == resourceOther || i > NUM_RESOURCE_TYPES)
+        if (i == resourceOther || i >= NUM_RESOURCE_TYPES)
             continue;
         page->resource_ids[i] = 0;
         for (j = 0; j < NUM_RESOURCE_CHAINS; ++j) {
@@ -1725,13 +1729,19 @@ pdf_copy_data_safe(stream *s, FILE *file, gs_offset_t position, long count)
         long copy = min(left, (long)sbuf_size);
         int64_t end_pos = gp_ftell_64(file);
 
-        gp_fseek_64(file, position + count - left, SEEK_SET);
+        if (gp_fseek_64(file, position + count - left, SEEK_SET) != 0) {
+            gs_note_error(gs_error_ioerror);
+            return;
+        }
         r = fread(buf, 1, copy, file);
         if (r < 1) {
             gs_note_error(gs_error_ioerror);
             return;
         }
-        gp_fseek_64(file, end_pos, SEEK_SET);
+        if (gp_fseek_64(file, end_pos, SEEK_SET) != 0) {
+            gs_note_error(gs_error_ioerror);
+            return;
+        }
         stream_write(s, buf, copy);
         sflush(s);
         left -= copy;
@@ -2222,7 +2232,7 @@ pdf_put_filters(cos_dict_t *pcd, gx_device_pdf *pdev, stream *s,
                 cos_dict_alloc(pdev, "pdf_put_image_filters(decode_parms)");
             if (decode_parms == 0)
                 return_error(gs_error_VMerror);
-            CHECK(cos_param_list_writer_init(&writer, decode_parms, 0));
+            CHECK(cos_param_list_writer_init(pdev, &writer, decode_parms, 0));
             /*
              * If EndOfBlock is true, we mustn't write a Rows value.
              * This is a hack....
@@ -2558,7 +2568,7 @@ pdf_function_aux(gx_device_pdf *pdev, const gs_function_t *pfn,
             return code;
         }
     }
-    code = cos_param_list_writer_init(&rlist, pcd, PRINT_BINARY_OK);
+    code = cos_param_list_writer_init(pdev, &rlist, pcd, PRINT_BINARY_OK);
     if (code < 0)
         return code;
     return gs_function_get_params(pfn, (gs_param_list *)&rlist);
@@ -2576,6 +2586,9 @@ pdf_function(gx_device_pdf *pdev, const gs_function_t *pfn, cos_value_t *pvalue)
 
     if (code < 0)
         return code;
+    if (pres->object->md5_valid)
+        pres->object->md5_valid = 0;
+
     code = pdf_substitute_resource(pdev, &pres, resourceFunction, functions_equal, false);
     if (code < 0)
         return code;
