@@ -558,6 +558,9 @@ pdf_process_string(pdf_text_enum_t *penum, gs_string *pstr,
             gs_point p0, p1, p2, p3;
 
             code = gx_path_current_point(penum->path, &origin);
+            if (code < 0)
+                return code;
+
             m = ctm_only(penum->pis);
             m.tx = fixed2float(origin.x);
             m.ty = fixed2float(origin.y);
@@ -1008,7 +1011,7 @@ process_text_return_width(const pdf_text_enum_t *pte, gs_font_base *font,
                 bbox->p.y = cw.BBox.p.y;
             if (cw.BBox.q.x > bbox->q.x)
                 bbox->q.x = cw.BBox.q.x;
-            if (cw.BBox.q.x > bbox->q.y)
+            if (cw.BBox.q.y > bbox->q.y)
                 bbox->q.y = cw.BBox.q.y;
         }
     }
