@@ -803,9 +803,12 @@ pclxl_write_image_data_JPEG(gx_device_pclxl * xdev, const byte * base,
     state.ColorTransform = (xdev->color_info.num_components == 3 ? 1 : 0);
     state.data.compress = jcdp;
     state.icc_profile = NULL;
-    jcdp->memory = state.jpeg_memory = xdev->v_memory;
+    /* state.memory needs set for creation..... */
+    state.memory = jcdp->memory = state.jpeg_memory = xdev->v_memory;
     if ((code = gs_jpeg_create_compress(&state)) < 0)
-	goto cleanup_and_use_rle;
+        goto cleanup_and_use_rle;
+    /* .... and NULL after, so we don't try to free the stack based "state" */
+    state.memory = NULL;
     /* image-specific info */
     jcdp->cinfo.image_width = width_bytes / xdev->color_info.num_components;
     jcdp->cinfo.image_height = height;
@@ -1549,8 +1552,8 @@ static const gx_device_vector_procs pclxl_vector_procs = {
 static int
 pclxl_open_device(gx_device * dev)
 {
-    gx_device_vector *const vdev = (gx_device_vector *)dev;
-    gx_device_pclxl *const xdev = (gx_device_pclxl *)dev;
+    gx_device_vector *vdev = (gx_device_vector *)dev;
+    gx_device_pclxl *xdev = (gx_device_pclxl *)dev;
     int code;
 
     vdev->v_memory = dev->memory;	/****** WRONG ******/
@@ -1560,6 +1563,11 @@ pclxl_open_device(gx_device * dev)
     if (code < 0)
         return code;
 
+    while (dev->child)
+        dev = dev->child;
+    vdev = (gx_device_vector *)dev;
+    xdev = (gx_device_pclxl *)dev;
+
     pclxl_page_init(xdev);
     px_write_file_header(vdev->strm, dev);
     xdev->media_size = pxeMediaSize_next;	/* no size selected */
