@@ -32,6 +32,8 @@
 #include "gxpath.h"
 #include "gdevagl.h"
 #include "gxdevsop.h"
+#include "gzpath.h"
+#include "gdevkrnlsclass.h" /* 'standard' built in subclasses, currently First/Last Page and obejct filter */
 
 /* #define TRACE_TXTWRITE 1 */
 
@@ -264,6 +266,7 @@ static int
 txtwrite_open_device(gx_device * dev)
 {
     gx_device_txtwrite_t *const tdev = (gx_device_txtwrite_t *) dev;
+    int code = 0;
 
     gx_device_fill_in_procs(dev);
     if (tdev->fname[0] == 0)
@@ -275,7 +278,9 @@ txtwrite_open_device(gx_device * dev)
 #ifdef TRACE_TXTWRITE
     tdev->DebugFile = gp_fopen("/temp/txtw_dbg.txt", "wb+");
 #endif
-    return 0;
+
+    code = install_internal_subclass_devices((gx_device **)&dev, NULL);
+    return code;
 }
 
 static int
@@ -676,7 +681,7 @@ static int decorated_text_output(gx_device_txtwrite_t *tdev)
             xpos = x_entry->start.x;
             for (i=0;i<x_entry->Unicode_Text_Size;i++) {
                 escaped_Unicode(x_entry->Unicode_Text[i], (char *)&Escaped);
-                gs_sprintf(TextBuffer, "<char bbox=\"%0.0f %0.0f %0.0f %0.0f\" c=\"%s\">\n", xpos,
+                gs_sprintf(TextBuffer, "<char bbox=\"%0.0f %0.0f %0.0f %0.0f\" c=\"%s\"/>\n", xpos,
                     x_entry->start.y, xpos + x_entry->Widths[i], x_entry->end.y, Escaped);
                 fwrite(TextBuffer, 1, strlen(TextBuffer), tdev->file);
                 xpos += x_entry->Widths[i];
@@ -799,7 +804,7 @@ static int decorated_text_output(gx_device_txtwrite_t *tdev)
                     xpos = x_entry->start.x;
                     for (i=0;i<x_entry->Unicode_Text_Size;i++) {
                         escaped_Unicode(x_entry->Unicode_Text[i], (char *)&Escaped);
-                        gs_sprintf(TextBuffer, "<char bbox=\"%0.0f %0.0f %0.0f %0.0f\" c=\"%s\">\n", xpos,
+                        gs_sprintf(TextBuffer, "<char bbox=\"%0.0f %0.0f %0.0f %0.0f\" c=\"%s\"/>\n", xpos,
                             x_entry->start.y, xpos + x_entry->Widths[i], x_entry->end.y, Escaped);
                         fwrite(TextBuffer, 1, strlen(TextBuffer), tdev->file);
                         xpos += x_entry->Widths[i];
@@ -1921,7 +1926,7 @@ txtwrite_process_plain_text(gs_text_enum_t *pte)
     gs_font *font = pte->orig_font;
     const gs_glyph *gdata = NULL;
     gs_glyph glyph;
-    gs_char ch;
+    gs_char ch = 0;
     int i, code;
     uint operation = pte->text.operation;
     txt_glyph_widths_t widths;
@@ -2166,6 +2171,9 @@ textw_text_process(gs_text_enum_t *pte)
     gs_font_base *font_base = (gs_font_base *)pte->current_font;
     int code = 0;
 
+    if (pte->text.size == 0)
+        return 0;
+
     if (!penum->TextBuffer) {
         /* We can get up to 4 Unicode points per glyph, and a glyph can be
          * be represented by as little as one byte. So we make a very large
@@ -2321,6 +2329,17 @@ txtwrite_text_begin(gx_device * dev, gs_imager_state * pis,
     textw_text_enum_t *penum;
     int code;
 
+    /* If this is a stringwidth, we must let the default graphics library code handle it
+     * in case there is no current point (this can happen if this is the first operation
+     * we get, the current font is a CIDFont, and its descendant is a substitute type 1
+     * font). If there is no current point we throw an error in text_process and that
+     * messes up all the font handling. The test below is copied from pdfwrite
+     * (gdev_pdf_text_begin) and seems to do the job.
+     */
+    if ((!(text->operation & TEXT_DO_DRAW) && pis->text_rendering_mode != 3)
+                    || path == 0 || !path_position_valid(path))
+            return gx_default_text_begin(dev, pis, text, font, path, pdcolor,
+                                         pcpath, mem, ppenum);
     /* Allocate and initialize one of our text enumerators. */
     rc_alloc_struct_1(penum, textw_text_enum_t, &st_textw_text_enum, mem,
                       return_error(gs_error_VMerror), "gdev_textw_text_begin");
@@ -2330,6 +2349,7 @@ txtwrite_text_begin(gx_device * dev, gs_imager_state * pis,
     penum->returned.total_width.x = penum->returned.total_width.y = 0;
     penum->TextBuffer = NULL;
     penum->TextBufferIndex = 0;
+    penum->Widths = NULL;
     /* The enumerator's text_release method frees this memory */
     penum->text_state = (text_list_entry_t *)gs_malloc(tdev->memory->stable_memory, 1,
             sizeof(text_list_entry_t), "txtwrite alloc text state");
