@@ -111,7 +111,7 @@ const char *opdfread_ps [] = {
 "/HexDigits(0123456789ABCDEF)readonly def\n",
 "/PrintHex\n",
 "{8{\n",
-"dup -28 bitshift//HexDigits exch 1 getinterval//=only exec\n",
+"dup -28 bitshift 15 and//HexDigits exch 1 getinterval//=only exec\n",
 "4 bitshift\n",
 "}repeat\n",
 "pop\n",
@@ -413,6 +413,7 @@ const char *opdfread_ps [] = {
 "//PDFR_DEBUG{\n",
 "(endobj )=\n",
 "}if\n",
+"count 1 eq {pop} {",
 "dup type/dicttype eq{\n",
 "dup/.endobj_daemon//knownget exec{\n",
 "//PDFR_DEBUG{(.endobj_daemon for )print 2 index =}if\n",
@@ -428,6 +429,7 @@ const char *opdfread_ps [] = {
 "//RunTypeDaemon exec\n",
 "//DoNothing 3 1 roll//Register exec\n",
 "}ifelse\n",
+"}ifelse",
 "}bind def\n",
 "/StoreBlock\n",
 "{\n",
@@ -583,7 +585,7 @@ const char *opdfread_ps [] = {
 "//PDFReader/InitialGraphicState get\n",
 "//PDFReader/GraphicState get copy pop\n",
 "//PDFReader/Operators get begin\n",
-"pop currentfile 0(endstream)/SubFileDecode filter\n",
+"currentfile exch ()/SubFileDecode filter\n",
 "1 index//AppendFilters exec\n",
 "cvx mark exch\n",
 "exec\n",
@@ -602,7 +604,7 @@ const char *opdfread_ps [] = {
 "copypage\n",
 "}repeat\n",
 "}if\n",
-"showpage\n",
+"EPS2Write not {showpage} if\n",
 "}if\n",
 "}bind def\n",
 "/stream\n",
@@ -734,10 +736,27 @@ const char *opdfread_ps [] = {
 "0 0}ifelse\n",
 "round cvi 2 index round cvi eq\n",
 "exch round cvi 3 index round cvi eq and{\n",
+"//PDFR_DEBUG{(PageSize matches request) == flush}if\n",
 "pop pop}{\n",
+"/MediaRequested where {\n",
+"//PDFR_DEBUG{(MediaRequested is true, check against new request) == flush}if\n",
+"/MediaRequested get aload pop\n",
+"round cvi 2 index round cvi eq\n",
+"exch round cvi 3 index round cvi eq and\n",
+"{//PDFR_DEBUG{(MediaRequested same as current request, ignore) == flush}if pop pop false}\n",
+"{//PDFR_DEBUG{(MediaRequested different to current request) == flush}if true}ifelse\n",
+"}{\n",
+"//PDFR_DEBUG{(No MediaRequested yet) == flush}if\n",
+"true\n",
+"}ifelse\n",
+"{\n",
+"//PDFR_DEBUG{(Setting pagesize) == flush}if\n",
 "2 array astore\n",
+"dup /MediaRequested exch def\n",
 "<< exch /PageSize exch >> setpagedevice\n",
-"} ifelse\n",
+"/pagesave save def\n",
+"}if\n",
+"}ifelse\n",
 "userdict/PDFR_InitialGS gstate put\n",
 "setglobal\n",
 "}if\n",
@@ -803,7 +822,7 @@ const char *opdfread_ps [] = {
 "1 index exch/Context exch put\n",
 "dup/ImmediateExec true put\n",
 "dup/IsPage true put\n",
-"dup/Context get//SetupPageView exec\n",
+"SetPageSize {dup/Context get//SetupPageView exec}if\n",
 "}bind def\n",
 "/FontFileDaemon\n",
 "{\n",
@@ -1142,11 +1161,14 @@ const char *opdfread_ps [] = {
 "/StringOffset StringOffset 4 add\n",
 "def\n",
 "}{\n",
-"dup\n",
-"StringOffset getu16\n",
-"LocaArray LocaIndex 3 -1 roll put\n",
+"StringOffset getu16 2 mul\n",
+"LocaArray length LocaIndex gt {",
+"LocaArray LocaIndex 3 -1 roll put",
+"}{",
+"pop",
+"}ifelse",
 "/LocaIndex LocaIndex 1 add def\n",
-"/StringOffset StringOffset 4 add\n",
+"/StringOffset StringOffset 2 add\n",
 "def\n",
 "}ifelse\n",
 "}{\n",
@@ -3137,6 +3159,7 @@ const char *opdfread_ps [] = {
 "}ifelse\n",
 "}if\n",
 "dup 0 get/Separation eq{\n",
+"dup dup 1 get UnPDFEscape 1 exch put\n",
 "3//ResolveArrayElement exec\n",
 "dup 3 get//FunctionToProc exec\n",
 "2 copy 3 exch put\n",
@@ -3903,9 +3926,9 @@ const char *opdfread_ps [] = {
 "/Tf//SetFont def\n",
 "/d0/setcharwidth load def\n",
 "/d1/setcachedevice load def\n",
-"/BDC{BeginMarkedContentSequenceWithPropertyList}bind def\n",
-"/BMC{BeginMarkedContentSequence}bind def\n",
-"/EMC{EndMarkedContentSequence}bind def\n",
+"/BDC{pop pop}bind def\n",
+"/BMC{pop}bind def\n",
+"/EMC{}bind def\n",
 "/BX{BeginCompatibilitySection}bind def\n",
 "/EX{EndCompatibilitySection}bind def\n",
 "/DP{DefineMarkedContentPointWithPropertyList}bind def\n",
