@@ -156,20 +156,20 @@ gs_main_init0(gs_main_instance * minst, FILE * in, FILE * out, FILE * err,
 #   else
     /* plmain settings remain in effect */
 #   endif
-    gp_get_realtime(minst->base_time);
+    gp_get_usertime(minst->base_time);
 
     /* Initialize the file search paths. */
     paths = (ref *) gs_alloc_byte_array(minst->heap, max_lib_paths, sizeof(ref),
                                         "lib_path array");
     if (paths == 0) {
-        gs_lib_finit(1, e_VMerror, minst->heap);
-        return_error(e_VMerror);
+        gs_lib_finit(1, gs_error_VMerror, minst->heap);
+        return_error(gs_error_VMerror);
     }
     array = (ref *) gs_alloc_byte_array(minst->heap, max_lib_paths, sizeof(ref),
                                         "lib_path array");
     if (array == 0) {
-        gs_lib_finit(1, e_VMerror, minst->heap);
-        return_error(e_VMerror);
+        gs_lib_finit(1, gs_error_VMerror, minst->heap);
+        return_error(gs_error_VMerror);
     }
     make_array(&minst->lib_path.container, avm_foreign, max_lib_paths,
                array);
@@ -188,7 +188,6 @@ int
 gs_main_init1(gs_main_instance * minst)
 {
     i_ctx_t *i_ctx_p;
-    extern init_proc(gs_iodev_init);
 
     if (minst->init_done < 1) {
         gs_dual_memory_t idmem;
@@ -208,7 +207,7 @@ gs_main_init1(gs_main_instance * minst)
                                         idmem.space_system);
 
             if (nt == 0)
-                return_error(e_VMerror);
+                return_error(gs_error_VMerror);
             mem->gs_lib_ctx->gs_name_table = nt;
             code = gs_register_struct_root(mem, NULL,
                                            (void **)&mem->gs_lib_ctx->gs_name_table,
@@ -223,7 +222,7 @@ gs_main_init1(gs_main_instance * minst)
         if (code < 0)
             return code;
         i_ctx_p = minst->i_ctx_p;
-        code = gs_iodev_init(imemory);
+        code = i_iodev_init(minst->i_ctx_p);
         if (code < 0)
             return code;
         minst->init_done = 1;
@@ -352,7 +351,7 @@ gs_main_init2(gs_main_instance * minst)
                 outprintf(minst->heap,
                           "   --saved-pages not supported by the '%s' device.\n",
                           pdev->dname);
-                return e_Fatal;
+                return gs_error_Fatal;
             }
             code = gx_saved_pages_param_process(ppdev, minst->saved_pages_initial_arg,
                                                 strlen(minst->saved_pages_initial_arg));
@@ -384,7 +383,7 @@ extend_path_list_container (gs_main_instance * minst, gs_file_path * pfp)
                                         "extend_path_list_container array");
 
     if (paths == 0) {
-        return_error(e_VMerror);
+        return_error(gs_error_VMerror);
     }
     make_array(&minst->lib_path.container, avm_foreign, len + LIB_PATH_EXTEND, paths);
     make_array(&minst->lib_path.list, avm_foreign | a_readonly, 0,
@@ -501,7 +500,12 @@ gs_main_set_lib_paths(gs_main_instance * minst)
         const char *dname = iodev->dname;
 
         if (dname && strlen(dname) == 5 && !memcmp("%rom%", dname, 5)) {
-            have_rom_device = 1;
+            struct stat pstat;
+            /* gs_error_unregistered means no usable romfs is available */
+            int code = iodev->procs.file_status((gx_io_device *)iodev, dname, &pstat);
+            if (code != gs_error_unregistered){
+                have_rom_device = 1;
+            }
             break;
         }
     }
@@ -553,7 +557,7 @@ gs_main_run_file_open(gs_main_instance * minst, const char *file_name, ref * pfr
         emprintf1(minst->heap,
                   "Can't find initialization file %s.\n",
                   file_name);
-        return_error(e_Fatal);
+        return_error(gs_error_Fatal);
     }
     r_set_attrs(pfref, a_execute + a_executable);
     return 0;
@@ -584,7 +588,7 @@ gs_run_init_file(gs_main_instance * minst, int *pexit_code, ref * perror_object)
                   "Initialization file %s does not begin with an integer.\n",
                   gs_init_file);
         *pexit_code = 255;
-        return_error(e_Fatal);
+        return_error(gs_error_Fatal);
     }
     *++osp = first_token;
     r_set_attrs(&ifile, a_executable);
@@ -613,7 +617,7 @@ gs_main_run_string_with_length(gs_main_instance * minst, const char *str,
         return code;
     code = gs_main_run_string_continue(minst, str, length, user_errors,
                                        pexit_code, perror_object);
-    if (code != e_NeedInput)
+    if (code != gs_error_NeedInput)
         return code;
     return gs_main_run_string_end(minst, user_errors,
                                   pexit_code, perror_object);
@@ -633,7 +637,7 @@ gs_main_run_string_begin(gs_main_instance * minst, int user_errors,
                       strlen(setup), (const byte *)setup);
     code = gs_main_interpret(minst, &rstr, user_errors, pexit_code,
                         perror_object);
-    return (code == e_NeedInput ? 0 : code == 0 ? e_Fatal : code);
+    return (code == gs_error_NeedInput ? 0 : code == 0 ? gs_error_Fatal : code);
 }
 /* Continue running a string with the option of suspending. */
 int
@@ -719,7 +723,7 @@ static int
 pop_value(i_ctx_t *i_ctx_p, ref * pvalue)
 {
     if (!ref_stack_count(&o_stack))
-        return_error(e_stackunderflow);
+        return_error(gs_error_stackunderflow);
     *pvalue = *ref_stack_index(&o_stack, 0L);
     return 0;
 }
@@ -771,7 +775,7 @@ gs_pop_real(gs_main_instance * minst, float *result)
             *result = (float)(vref.value.intval);
             break;
         default:
-            return_error(e_typecheck);
+            return_error(gs_error_typecheck);
     }
     ref_stack_pop(&o_stack, 1);
     return 0;
@@ -797,7 +801,7 @@ gs_pop_string(gs_main_instance * minst, gs_string * result)
             result->size = r_size(&vref);
             break;
         default:
-            return_error(e_typecheck);
+            return_error(gs_error_typecheck);
     }
     ref_stack_pop(&o_stack, 1);
     return code;
@@ -906,7 +910,7 @@ gs_main_finit(gs_main_instance * minst, int exit_status, int code)
                 emprintf1(minst->heap,
                           "ERROR %d reclaiming the memory while the interpreter finalization.\n",
                           code);
-                return e_Fatal;
+                return gs_error_Fatal;
             }
             i_ctx_p = minst->i_ctx_p; /* interp_reclaim could change it. */
         }
@@ -930,7 +934,7 @@ gs_main_finit(gs_main_instance * minst, int exit_status, int code)
                           code,
                           dname);
             rc_decrement(pdev, "gs_main_finit");                /* device might be freed */
-            if (exit_status == 0 || exit_status == e_Quit)
+            if (exit_status == 0 || exit_status == gs_error_Quit)
                 exit_status = code;
         }
 #endif
@@ -1016,7 +1020,7 @@ print_resource_usage(const gs_main_instance * minst, gs_dual_memory_t * dmem,
     ulong allocated = 0, used = 0;
     long utime[2];
 
-    gp_get_realtime(utime);
+    gp_get_usertime(utime);
     {
         int i;
 
