@@ -106,7 +106,7 @@ zbuildsampledfunction(i_ctx_t *i_ctx_p)
      * Check procedure to be sampled.
      */
     if (dict_find_string(pdict, "Function", &pfunc) <= 0)
-        return_error(e_rangecheck);
+        return_error(gs_error_rangecheck);
     check_proc(*pfunc);
     /*
      * Set up the hyper cube function data structure.
@@ -163,7 +163,7 @@ valid_cube_size(int num_inputs, int num_outputs, int sample_size, const int Size
  *
  * We do check to see if the data will fit using our initial guess.  If not
  * then we decrement the size of each edge until it fits.  We will return a
- * e_rangecheck error if the cube can not fit into the maximum  size.
+ * gs_error_rangecheck error if the cube can not fit into the maximum  size.
  * On exit the Size array contains the cube size (if a valid size was found).
  */
 static int
@@ -301,7 +301,7 @@ cube_build_func0(const ref * pdict, gs_function_Sd_params_t * params,
     params->n >>= 1;
     if (params->m == 0 || params->n == 0 ||
         params->m > MAX_NUM_INPUTS || params->n > MAX_NUM_OUTPUTS) {
-        code = gs_note_error(e_rangecheck);
+        code = gs_note_error(gs_error_rangecheck);
         goto fail;
     }
     /*
@@ -313,7 +313,7 @@ cube_build_func0(const ref * pdict, gs_function_Sd_params_t * params,
             gs_alloc_byte_array(mem, params->m, sizeof(int), "Size");
 
         if (ptr == NULL) {
-            code = gs_note_error(e_VMerror);
+            code = gs_note_error(gs_error_VMerror);
             goto fail;
         }
         params->Size = ptr;
@@ -333,7 +333,7 @@ cube_build_func0(const ref * pdict, gs_function_Sd_params_t * params,
         else {			/* Size array specified - verify valid */
             if (code != params->m || !valid_cube_size(params->m, params->n,
                                         params->BitsPerSample, params->Size))
-                code = gs_note_error(e_rangecheck);
+                code = gs_note_error(gs_error_rangecheck);
                 goto fail;
         }
     }
@@ -348,7 +348,7 @@ cube_build_func0(const ref * pdict, gs_function_Sd_params_t * params,
      */
     bytes = gs_alloc_byte_array(mem, total_size, 1, "cube_build_func0(bytes)");
     if (!bytes) {
-        code = gs_note_error(e_VMerror);
+        code = gs_note_error(gs_error_VMerror);
         goto fail;
     }
     data_source_init_bytes(&params->DataSource,
@@ -358,7 +358,7 @@ cube_build_func0(const ref * pdict, gs_function_Sd_params_t * params,
 
 fail:
     gs_function_Sd_free_params(params, mem);
-    return (code < 0 ? code : gs_note_error(e_rangecheck));
+    return (code < 0 ? code : gs_note_error(gs_error_rangecheck));
 }
 
 /*
@@ -407,7 +407,7 @@ sampled_data_setup(i_ctx_t *i_ctx_p, gs_function_t *pfn,
      */
     penum = gs_sampled_data_enum_alloc(imemory, "zbuildsampledfuntion(params)");
     if (penum == NULL)
-        return_error(e_VMerror);
+        return_error(gs_error_VMerror);
 
     /* Initialize data in the enumeration structure */
 
@@ -506,7 +506,7 @@ sampled_data_continue(i_ctx_t *i_ctx_p)
             push(-stack_depth_adjust);
             ifree_object(penum->pfn, "sampled_data_continue(pfn)");
             ifree_object(penum, "sampled_data_continue((enum)");
-            return_error(e_undefinedresult);
+            return_error(gs_error_undefinedresult);
         }
     }
 
@@ -606,7 +606,7 @@ int make_sampled_function(i_ctx_t * i_ctx_p, ref *arr, ref *pproc, gs_function_t
     if (code < 0)
         return code;
     if (!space->alternateproc)
-        return e_typecheck;
+        return gs_error_typecheck;
     code = space->alternateproc(i_ctx_p, arr, &palternatespace, &CIESubst);
     if (code < 0)
         return code;
@@ -624,7 +624,7 @@ int make_sampled_function(i_ctx_t * i_ctx_p, ref *arr, ref *pproc, gs_function_t
         return code;
     fptr = (float *)gs_alloc_byte_array(imemory, num_components * 2, sizeof(float), "make_sampled_function(Domain)");
     if (!fptr)
-        return e_VMerror;
+        return gs_error_VMerror;
     code = space->domain(i_ctx_p, arr, fptr);
     if (code < 0) {
         gs_free_const_object(imemory, fptr, "make_sampled_function(Domain)");
@@ -641,7 +641,7 @@ int make_sampled_function(i_ctx_t * i_ctx_p, ref *arr, ref *pproc, gs_function_t
     fptr = (float *)gs_alloc_byte_array(imemory, num_components * 2, sizeof(float), "make_sampled_function(Range)");
     if (!fptr) {
         gs_free_const_object(imemory, params.Domain, "make_sampled_function(Domain)");
-        return e_VMerror;
+        return gs_error_VMerror;
     }
     code = altspace->range(i_ctx_p, palternatespace, fptr);
     if (code < 0) {
@@ -658,7 +658,7 @@ int make_sampled_function(i_ctx_t * i_ctx_p, ref *arr, ref *pproc, gs_function_t
      */
     ptr = (int *)gs_alloc_byte_array(imemory, params.m, sizeof(int), "Size");
     if (ptr == NULL) {
-        code = gs_note_error(e_VMerror);
+        code = gs_note_error(gs_error_VMerror);
         goto fail;
     }
     params.Size = ptr;
@@ -681,7 +681,7 @@ int make_sampled_function(i_ctx_t * i_ctx_p, ref *arr, ref *pproc, gs_function_t
      */
     bytes = gs_alloc_byte_array(imemory, total_size, 1, "cube_build_func0(bytes)");
     if (!bytes) {
-        code = gs_note_error(e_VMerror);
+        code = gs_note_error(gs_error_VMerror);
         goto fail;
     }
     data_source_init_bytes(&params.DataSource,
@@ -705,7 +705,7 @@ int make_sampled_function(i_ctx_t * i_ctx_p, ref *arr, ref *pproc, gs_function_t
 
 fail:
     gs_function_Sd_free_params(&params, imemory);
-    return (code < 0 ? code : gs_note_error(e_rangecheck));
+    return (code < 0 ? code : gs_note_error(gs_error_rangecheck));
 }
 
 /* ------ Initialization procedure ------ */
