@@ -75,7 +75,7 @@ int_pattern_alloc(int_pattern **ppdata, const ref *op, gs_memory_t *mem)
         gs_alloc_struct(mem, int_pattern, &st_int_pattern, "int_pattern");
 
     if (pdata == 0)
-        return_error(e_VMerror);
+        return_error(gs_error_VMerror);
     pdata->dict = *op;
     *ppdata = pdata;
     return 0;
@@ -106,7 +106,7 @@ zbuildpattern1(i_ctx_t *i_ctx_p)
     if (code < 0)
         return code;
     if (code != 1)
-        return_error(e_rangecheck);
+        return_error(gs_error_rangecheck);
 
     code = dict_int_param(op1, "PaintType", 1, 2, 0, &templat.PaintType);
     if (code < 0)
@@ -124,32 +124,32 @@ zbuildpattern1(i_ctx_t *i_ctx_p)
     if (code < 0)
         return code;
     if (code == 0)
-       return_error(e_undefined);
+       return_error(gs_error_undefined);
 
     code = dict_float_param(op1, "XStep", 0.0, &templat.XStep);
     if (code < 0)
         return code;
     if (code == 1)
-       return_error(e_undefined);
+       return_error(gs_error_undefined);
 
     code = dict_float_param(op1, "YStep", 0.0, &templat.YStep);
     if (code < 0)
         return code;
     if (code == 1)
-       return_error(e_undefined);
+       return_error(gs_error_undefined);
 
     code = dict_find_string(op1, "PaintProc", &pPaintProc);
     if (code < 0)
         return code;
     if (code == 0)
-       return_error(e_undefined);
+       return_error(gs_error_undefined);
 
     check_proc(*pPaintProc);
 
     if (mat.xx * mat.yy == mat.xy * mat.yx)
-        return_error(e_undefinedresult);
+        return_error(gs_error_undefinedresult);
     if (BBox[0] >= BBox[2] ||  BBox[1] >= BBox[3])
-        return_error(e_rangecheck);
+        return_error(gs_error_rangecheck);
 
     templat.BBox.p.x = BBox[0];
     templat.BBox.p.y = BBox[1];
@@ -191,7 +191,7 @@ zPaintProc(const gs_client_color * pcc, gs_state * pgs)
     r_ptr(&gs_int_gstate(pgs)->remap_color_info,
           int_remap_color_info_t)->proc =
         pattern_paint_prepare;
-    return_error(e_RemapColor);
+    return_error(gs_error_Remap_Color);
 }
 /* Prepare to run the PaintProc. */
 static int
@@ -220,7 +220,7 @@ pattern_paint_prepare(i_ctx_t *i_ctx_p)
 
         pdev = gx_pattern_accum_alloc(imemory, storage_memory, pinst, "pattern_paint_prepare");
         if (pdev == 0)
-            return_error(e_VMerror);
+            return_error(gs_error_VMerror);
         code = (*dev_proc(pdev, open_device)) ((gx_device *) pdev);
         if (code < 0) {
             ifree_object(pdev, "pattern_paint_prepare");
@@ -273,8 +273,17 @@ pattern_paint_prepare(i_ctx_t *i_ctx_p)
             gs_grestore(pgs);
             return code;
         }
-        code = dev_proc(cdev, dev_spec_op)(cdev, gxdso_pattern_start_accum,
-                                pinst, pinst->id);
+
+        {
+            pattern_accum_param_s param;
+            param.pinst = (void *)pinst;
+            param.graphics_state = (void *)pgs;
+            param.pinst_id = pinst->id;
+
+            code = (*dev_proc(pgs->device, dev_spec_op))((gx_device *)pgs->device,
+                gxdso_pattern_start_accum, &param, sizeof(pattern_accum_param_s));
+        }
+
         if (code < 0) {
             gs_grestore(pgs);
             return code;
@@ -345,21 +354,25 @@ pattern_paint_cleanup(i_ctx_t *i_ctx_p)
 {
     gx_device_pattern_accum *const pdev =
         r_ptr(esp + 2, gx_device_pattern_accum);
+        gs_pattern1_instance_t *pinst = (gs_pattern1_instance_t *)gs_currentcolor(igs->saved)->pattern;
     int code;
 
     if (pdev != NULL) {
         /* grestore will free the device, so close it first. */
         (*dev_proc(pdev, close_device)) ((gx_device *) pdev);
     }
-    code = gs_grestore(igs);
-    gx_unset_dev_color(igs);	/* dev_color may need updating if GC ran */
     if (pdev == NULL) {
         gx_device *cdev = gs_currentdevice_inline(igs);
-        int code1 = dev_proc(cdev, dev_spec_op)(cdev,
-                        gxdso_pattern_finish_accum, NULL, gx_no_bitmap_id);
+        pattern_accum_param_s param;
 
-        if (code == 0 && code1 < 0)
-            code = code1;
+        param.pinst = (void *)pinst;
+        param.graphics_state = (void *)igs;
+        param.pinst_id = pinst->id;
+
+        code = dev_proc(cdev, dev_spec_op)(cdev,
+                        gxdso_pattern_finish_accum, &param, sizeof(pattern_accum_param_s));
     }
+    code = gs_grestore(igs);
+    gx_unset_dev_color(igs);	/* dev_color may need updating if GC ran */
     return code;
 }
