@@ -36,11 +36,12 @@
 #include "gdevdevn.h"
 #include "gxblend.h"
 #include "gdevp14.h"
+#include "gsicc_cms.h"
 
 /* ------ Utilities ------ */
 
 static int
-set_float_value(i_ctx_t *i_ctx_p, int (*set_value)(gs_state *, floatp))
+set_float_value(i_ctx_t *i_ctx_p, int (*set_value)(gs_state *, double))
 {
     os_ptr op = osp;
     double value;
@@ -78,7 +79,7 @@ enum_param(const gs_memory_t *mem, const ref *pnref,
             !memcmp(*p, nsref.value.const_bytes, r_size(&nsref))
             )
             return p - names;
-    return_error(e_rangecheck);
+    return_error(gs_error_rangecheck);
 }
 
 /* ------ Graphics state operators ------ */
@@ -228,8 +229,19 @@ zbegintransparencygroup(i_ctx_t *i_ctx_p)
     if (dict_find_string(dop, "CS", &dummy) <= 0) {
         params.ColorSpace = NULL;
     } else {
-        /* the PDF interpreter set the colorspace, so use it */
+        /* the PDF interpreter sets the colorspace, so use it */
         params.ColorSpace = gs_currentcolorspace(igs);
+        /* Lets make sure that it is not an ICC color space that came from 
+           a PS CIE color space or a PS color space. These are 1-way color
+           spaces and cannot be used for group color spaces */
+        if (gs_color_space_is_PSCIE(params.ColorSpace))
+            params.ColorSpace = NULL;
+        else if (gs_color_space_is_ICC(params.ColorSpace) &&
+            params.ColorSpace->cmm_icc_profile_data != NULL &&
+            params.ColorSpace->cmm_icc_profile_data->profile_handle != NULL) {
+            if (gscms_is_input(params.ColorSpace->cmm_icc_profile_data->profile_handle))
+                params.ColorSpace = NULL;
+        }
     }
     code = gs_begin_transparency_group(igs, &params, &bbox);
     if (code < 0)
@@ -247,7 +259,7 @@ zendtransparencygroup(i_ctx_t *i_ctx_p)
 
 /* <cs_set?> <paramdict> <llx> <lly> <urx> <ury> .begintransparencymaskgroup -	*/
 /*             cs_set == false if we are inheriting the colorspace		*/
-static int tf_using_function(floatp, float *, void *);
+static int tf_using_function(double, float *, void *);
 static int
 zbegintransparencymaskgroup(i_ctx_t *i_ctx_p)
 {
@@ -264,7 +276,7 @@ zbegintransparencymaskgroup(i_ctx_t *i_ctx_p)
     check_type(*dop, t_dictionary);
     check_dict_read(*dop);
     if (dict_find_string(dop, "Subtype", &pparam) <= 0)
-        return_error(e_rangecheck);
+        return_error(gs_error_rangecheck);
     if ((code = enum_param(imemory, pparam, subtype_names)) < 0)
         return code;
     gs_trans_mask_params_init(&params, code);
@@ -282,7 +294,7 @@ zbegintransparencymaskgroup(i_ctx_t *i_ctx_p)
         gs_function_t *pfn = ref_function(pparam);
 
         if (pfn == 0 || pfn->params.m != 1 || pfn->params.n != 1)
-            return_error(e_rangecheck);
+            return_error(gs_error_rangecheck);
         params.TransferFunction = tf_using_function;
         params.TransferFunction_data = pfn;
     }
@@ -292,6 +304,17 @@ zbegintransparencymaskgroup(i_ctx_t *i_ctx_p)
     /* Is the colorspace set for this mask ? */
     if (op[-5].value.boolval) {
                 params.ColorSpace = gs_currentcolorspace(igs);
+                /* Lets make sure that it is not an ICC color space that came from
+                a PS CIE color space or a PS color space. These are 1-way color
+                spaces and cannot be used for group color spaces */
+                if (gs_color_space_is_PSCIE(params.ColorSpace))
+                    params.ColorSpace = NULL;
+                else if (gs_color_space_is_ICC(params.ColorSpace) &&
+                    params.ColorSpace->cmm_icc_profile_data != NULL &&
+                    params.ColorSpace->cmm_icc_profile_data->profile_handle != NULL) {
+                    if (gscms_is_input(params.ColorSpace->cmm_icc_profile_data->profile_handle))
+                        params.ColorSpace = NULL;
+                }
     } else {
         params.ColorSpace = NULL;
     }
@@ -312,7 +335,7 @@ zbegintransparencymaskimage(i_ctx_t *i_ctx_p)
     gs_color_space *gray_cs = gs_cspace_new_DeviceGray(imemory);
 
     if (!gray_cs)
-        return_error(e_VMerror);
+        return_error(gs_error_VMerror);
     gs_trans_mask_params_init(&params, TRANSPARENCY_MASK_Luminosity);
     code = gs_begin_transparency_mask(igs, &params, &bbox, true);
     if (code < 0)
@@ -323,7 +346,7 @@ zbegintransparencymaskimage(i_ctx_t *i_ctx_p)
 
 /* Implement the TransferFunction using a Function. */
 static int
-tf_using_function(floatp in_val, float *out, void *proc_data)
+tf_using_function(double in_val, float *out, void *proc_data)
 {
     float in = in_val;
     gs_function_t *const pfn = proc_data;
@@ -361,7 +384,7 @@ zimage3x(i_ctx_t *i_ctx_p)
     memset(&image, 0, sizeof(gs_image3x_t));
     gs_image3x_t_init(&image, NULL);
     if (dict_find_string(op, "DataDict", &pDataDict) <= 0)
-        return_error(e_rangecheck);
+        return_error(gs_error_rangecheck);
     if ((code = pixel_image_params(i_ctx_p, pDataDict,
                    (gs_pixel_image_t *)&image, &ip_data,
                    16, false, gs_currentcolorspace(igs))) < 0 ||
@@ -415,7 +438,7 @@ image_params *pip_data, const char *dict_name,
         ip_mask.MultipleDataSources ||
         mcode != (pixm->InterleaveType != 3)
         )
-        return_error(e_rangecheck);
+        return_error(gs_error_rangecheck);
     if (pixm->InterleaveType == 3) {
         /* Insert the mask DataSource before the data DataSources. */
         memmove(&pip_data->DataSource[1], &pip_data->DataSource[0],
@@ -451,6 +474,13 @@ zpoppdf14devicefilter(i_ctx_t *i_ctx_p)
     return gs_pop_pdf14trans_device(igs, false);
 }
 
+/* Something has gone terribly wrong */
+static int
+zabortpdf14devicefilter(i_ctx_t *i_ctx_p)
+{
+    return gs_abort_pdf14trans_device(igs);
+}
+
 /* This is used to communicate to the transparency compositor
    when a q (save extended graphic state) occurs.  Since
    the softmask is part of the graphic state we need to know
@@ -502,5 +532,6 @@ const op_def ztrans2_op_defs[] = {
     {"1.image3x", zimage3x},
     {"1.pushpdf14devicefilter", zpushpdf14devicefilter},
     {"0.poppdf14devicefilter", zpoppdf14devicefilter},
+    {"0.abortpdf14devicefilter", zabortpdf14devicefilter},
     op_def_end(0)
 };
