@@ -1,24 +1,20 @@
 #!/usr/bin/tclsh
 
-#  Copyright (C) 1997, 2000 artofcode LLC.  All rights reserved.
+# Copyright (C) 2001-2012 Artifex Software, Inc.
+# All Rights Reserved.
+#
+# This software is provided AS-IS with no warranty, either express or
+# implied.
+#
+# This software is distributed under license and may not be copied,
+# modified or distributed except as expressly authorized under the terms
+# of the license contained in the file LICENSE in this distribution.
+#
+# Refer to licensing information at http://www.artifex.com or contact
+# Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
+# CA  94903, U.S.A., +1(415)492-9861, for further information.
 #
-#  This file is part of GNU Ghostscript.
-# 
-#  GNU Ghostscript is distributed in the hope that it will be useful, but
-#  WITHOUT ANY WARRANTY.  No author or distributor accepts responsibility
-#  to anyone for the consequences of using it or for whether it serves any
-#  particular purpose or works at all, unless he says so in writing.  Refer
-#  to the GNU General Public License for full details.
-# 
-#  Everyone is granted permission to copy, modify and redistribute GNU
-#  Ghostscript, but only under the conditions described in the GNU General
-#  Public License.  A copy of this license is supposed to have been given
-#  to you along with GNU Ghostscript so you can know your rights and
-#  responsibilities.  It should be in a file named COPYING.  Among other
-#  things, the copyright notice and this notice must be preserved on all
-#  copies.
 
-# $Id: leaks.tcl,v 1.3.2.1 2001/11/02 23:05:44 giles Exp $
 
 # This tool helps detect memory leaks in a -ZA trace from Ghostscript.
 # It reads a memory trace from stdin and prints unmatched allocations on
@@ -77,6 +73,7 @@ proc add0-1 {il addr} [info body add0+0]
 proc read_trace {{fname %stdin}} {
     global A lines next
     set n $next
+    set i 0
     if {$fname == "%stdin"} {
 	set in stdin
     } else {
@@ -85,10 +82,11 @@ proc read_trace {{fname %stdin}} {
     # Skip to the first "allocated" line.  See below for why we bother
     # checking for EOF.
     while {[gets $in l] >= 0} {
+        incr i
 	if [regexp "memory allocated" $l] break
 	incr n
     }
-    if {$n == 0} {
+    if {$i == 0} {
 	puts stderr "Empty input file!"
 	if {$fname != "%stdin"} {close $in}
 	exit
